﻿Imports System.Text.RegularExpressions
Imports Scoreboard.My.Resources

Public Class Anzeige
    Private currentPeriod As Integer = 1
    Dim currentTime As Integer = 0
    Dim currentTimeout As Integer = 0
    Dim _homePoints As Integer = 0
    Dim _guestPoints As Integer = 0
    Dim _extraTimeRound As Integer = 0
    Dim _ownerForm As Bedienung = Nothing

    Public ReadOnly Property GetCurrentGameTime() As Integer
        Get
            Return currentTime
        End Get
    End Property

    Public Property HomePoints As Integer
        Get
            Return _homePoints
        End Get
        Set(value As Integer)
            _homePoints = value
            RaiseEvent OnSavePointsComplete(Team.Home, _homePoints)
        End Set
    End Property

    Public Property GuestPoints As Integer
        Get
            Return _guestPoints
        End Get
        Set(value As Integer)
            _guestPoints = value
            RaiseEvent OnSavePointsComplete(Team.Guest, _guestPoints)
        End Set
    End Property

    Public ReadOnly Property GetPeriod() As Integer
        Get
            Return currentPeriod
        End Get
    End Property

    Public Event OnStartGameTimer()
    Public Event OnGameTimerTick(ByVal curentTime As Integer)
    Public Event OnTimerEnd(ByVal isHalf As Boolean)
    Public Event OnTimeoutCompleted()
    Public Event OnResetMatch()
    Public Event OnSavePointsComplete(selectedTeam As Integer, points As Integer)

    Private Sub Anzeige_Load(sender As Object, e As EventArgs) Handles MyBase.Load
#If DEBUG Then
        Me.GameTimer.Interval = 10
        Me.TimeOutTimer.Interval = 50
#End If
        _ownerForm = Me.Owner
        AddHandler _ownerForm.GamePanel().OnRaiseAddScore, AddressOf OnAddScoreUI
        AddHandler _ownerForm.GamePanel().OnRaiseSubScore, AddressOf OnSubScoreUI
        AddHandler _ownerForm.GamePanel().OnRaiseToggleClock, AddressOf OnStartStopGameTimer
        AddHandler _ownerForm.GamePanel().OnRaiseStopClock, AddressOf OnStopGameTimer
        AddHandler _ownerForm.OnStopGameTimer, AddressOf OnStopGameTimer
        AddHandler _ownerForm.GamePanel().OnRaiseTimeout, AddressOf OnTeamTimeout
        AddHandler _ownerForm.GamePanel().OnChangeCurrentGameTimeByUser, AddressOf OnChangeCurrentGameTime
        AddHandler _ownerForm.OnChangeTeamName, AddressOf ChangeDisplayTeamName
        AddHandler _ownerForm.OnChangeGameTitle, AddressOf ChangeGameTitle
        AddHandler _ownerForm.OnChangeInfoText, AddressOf ChangeInfoText
        AddHandler _ownerForm.OnSpacePressed, AddressOf OnStartStopGameTimer
        AddHandler _ownerForm.OnNumPlusPressed, AddressOf OnAddScoreUI
        AddHandler _ownerForm.OnNumMinusPressed, AddressOf OnSubScoreUI
        AddHandler _ownerForm.OnPlusPressed, AddressOf OnAddScoreUI
        AddHandler _ownerForm.OnMinusPressed, AddressOf OnSubScoreUI
        AddHandler Me.OnGameTimerTick, AddressOf ChangeTimerUI
        AddHandler _ownerForm.OnAddPenalty2BindingSourceComplete, AddressOf ChangePenaltyUI
        AddHandler _ownerForm.OnEditPenaltyComplete, AddressOf ChangePenaltyUI
        AddHandler OnTimerEnd, AddressOf OnGameTimerEnd
        AddHandler OnResetMatch, AddressOf ResetGameData
        Me.ChangePeriodUI()
    End Sub

    Private Sub OnStartStopGameTimer()
        If My.Settings.gameTime = currentTime Then
            If Not My.Settings.cupMatch Then
                Dim dlgResult As DialogResult = MessageBox.Show(GlobalStrings.NewGameMessage, GlobalStrings.FullTimeMatchEnd, MessageBoxButtons.YesNo)

                If dlgResult = DialogResult.Yes Then
                    RaiseEvent OnResetMatch()
                    Return
                Else
                    Return
                End If
            End If
        End If
        If Me.GameTimer.Enabled Then
            Me.GameTimer.Stop()
            _ownerForm.GamePanel().TimerStopped = True
        Else
            Me.GameTimer.Start()
            _ownerForm.GamePanel().TimerStopped = False
        End If
    End Sub

    Private Sub OnStopGameTimer()
        Me.GameTimer.Stop()
        _ownerForm.GamePanel().TimerStopped = True
    End Sub

    Private Sub OnChangeCurrentGameTime(e As GameTimeEventArgs)
        currentTime = e.NewGameTime
        Me.ChangeTimerUI(currentTime)
    End Sub

    Private Sub OnAddScoreUI(selectedTeam As Integer)
        Select Case selectedTeam
            Case Team.Home
                If Me.HomePoints = 99 Then
                    Return
                End If
                Me.HomePoints += 1
            Case Team.Guest
                If Me.GuestPoints = 99 Then
                    Return
                End If
                Me.GuestPoints += 1
        End Select
    End Sub

    Private Sub OnSubScoreUI(selectedTeam As Integer)
        Select Case selectedTeam
            Case Team.Home
                If Me.HomePoints = 0 Then
                    Return
                End If
                Me.HomePoints -= 1
            Case Team.Guest
                If Me.GuestPoints = 0 Then
                    Return
                End If
                Me.GuestPoints -= 1
        End Select
    End Sub

    Private Sub GameTimer_Tick(sender As Object, e As EventArgs) Handles GameTimer.Tick
        currentTime += 1000
        Dim _TimerOnEnd As Boolean = False
        Dim _isHalfTime As Boolean = False
        If currentTime >= 5999000 Then
            Me.GameTimer.Stop()
            _ownerForm.GamePanel().TimerStopped = True
            _TimerOnEnd = True
            _isHalfTime = False
            MsgBox("Maximale Zeitanzeige erreicht")
        End If
        If My.Settings.halfTime = currentTime Then
            Me.GameTimer.Stop()
            _ownerForm.GamePanel().TimerStopped = True
            _TimerOnEnd = True
            _isHalfTime = True
        End If

        If My.Settings.gameTime = currentTime Then
            Me.GameTimer.Stop()
            _ownerForm.GamePanel().TimerStopped = True
            _TimerOnEnd = True
            _isHalfTime = False

        End If

        If My.Settings.cupMatch = True And _extraTimeRound > 0 Then
            Dim extrahalftime As Integer = ((My.Settings.extraTime * _extraTimeRound) - My.Settings.extraHalftime) + My.Settings.gameTime
            Dim extrafulltime As Integer = (My.Settings.extraTime * _extraTimeRound) + My.Settings.gameTime
            If extrahalftime = currentTime Then
                Me.GameTimer.Stop()
                _ownerForm.GamePanel().TimerStopped = True
                _TimerOnEnd = True
                _isHalfTime = True
            End If

            If extrafulltime = currentTime Then
                Me.GameTimer.Stop()
                _ownerForm.GamePanel().TimerStopped = True
                _TimerOnEnd = True
                _isHalfTime = False
            End If
        End If
        RaiseEvent OnGameTimerTick(currentTime)
        If _TimerOnEnd Then
            RaiseEvent OnTimerEnd(_isHalfTime)
        End If
    End Sub

    Private Sub ChangeTimerUI(currentTime As Integer)

        Dim Minutes As Integer = 0
        Dim Seconds As Integer = currentTime / 1000
        If (Seconds > 59) Then
            Dim _convTime2Minutes = Seconds / 60
            Minutes = Convert.ToInt16(_convTime2Minutes.ToString.Split(",")(0))
            Seconds -= (Minutes * 60)
        End If

        Select Case Seconds
            Case < 10
                With Me.GameTimeSecondRightPB
                    .Image = My.Resources.ResourceManager.GetObject("d_" & Seconds.ToString & "_red")
                End With
                With Me.GameTimeSecondLeftPB
                    .Image = My.Resources.ResourceManager.GetObject("d_0_red")
                End With
            Case 10 To 59
                Dim digitLeft = Seconds.ToString.Substring(0, 1)
                Dim digitRight = Seconds.ToString.Substring(1, 1)
                With Me.GameTimeSecondRightPB
                    .Image = My.Resources.ResourceManager.GetObject("d_" & digitRight & "_red")
                End With
                With Me.GameTimeSecondLeftPB
                    .Image = My.Resources.ResourceManager.GetObject("d_" & digitLeft & "_red")
                End With
        End Select
        Select Case Minutes
            Case < 10
                With Me.GameTimeMinuteRightPB
                    .Image = My.Resources.ResourceManager.GetObject("d_" & Minutes.ToString & "_red")
                End With
                With Me.GameTimeMinuteLeftPB
                    .Image = My.Resources.ResourceManager.GetObject("d_0_red")
                End With
            Case 10 To 99
                Dim digitLeft = Minutes.ToString.Substring(0, 1)
                Dim digitRight = Minutes.ToString.Substring(1, 1)
                With Me.GameTimeMinuteRightPB
                    .Image = My.Resources.ResourceManager.GetObject("d_" & digitRight & "_red")
                End With
                With Me.GameTimeMinuteLeftPB
                    .Image = My.Resources.ResourceManager.GetObject("d_" & digitLeft & "_red")
                End With
            Case Else
                'Throw New IndexOutOfRangeException("Display could show only 99 Minutes or less")
        End Select
    End Sub

    Public Sub ChangePenaltyUI()
        Dim homePenalties As Integer = CType(_ownerForm, Bedienung).PenaltyHomeBS.List.Count
        Dim guestPenalties As Integer = CType(_ownerForm, Bedienung).PenaltyGuestBS.List.Count

        ' Change Panalty UI for Home Team
        Select Case homePenalties
            Case 0
                HomePenaltyPanel1.Controls.Clear()
                HomePenaltyPanel2.Controls.Clear()
                HomePenaltyPanel3.Controls.Clear()
            Case < 2
                With HomePenaltyPanel1
                    Dim ctrl As PenaltyControl
                    Dim source As Penalty = CType(_ownerForm, Bedienung).PenaltyHomeBS.List(0)
                    Dim ctrls = .Controls
                    Select Case ctrls.Count
                        Case 0
                            ctrl = New PenaltyControl(source.PlayerNumber, source.PenaltyTime, source.PlayerName)
                            .Controls.Add(ctrl)
                        Case 1
                            ctrl = ctrls(0)
                            With ctrl
                                .PenaltyTime = source.PenaltyTime
                                .PlayerNumber = source.PlayerNumber
                                .PlayerName = source.PlayerName
                            End With
                    End Select
                End With
                HomePenaltyPanel2.Controls.Clear()
            Case < 3
                With HomePenaltyPanel1
                    Dim ctrl As PenaltyControl
                    Dim source As Penalty = CType(_ownerForm, Bedienung).PenaltyHomeBS.List(0)
                    Dim ctrls = .Controls
                    Select Case ctrls.Count
                        Case 0
                            ctrl = New PenaltyControl(source.PlayerNumber, source.PenaltyTime, source.PlayerName)
                            .Controls.Add(ctrl)
                        Case 1
                            ctrl = ctrls(0)
                            With ctrl
                                .PenaltyTime = source.PenaltyTime
                                .PlayerNumber = source.PlayerNumber
                                .PlayerName = source.PlayerName
                            End With
                    End Select
                End With
                With HomePenaltyPanel2
                    Dim ctrl As PenaltyControl
                    Dim source As Penalty = CType(_ownerForm, Bedienung).PenaltyHomeBS.List(1)
                    Dim ctrls = .Controls
                    Select Case ctrls.Count
                        Case 0
                            ctrl = New PenaltyControl(source.PlayerNumber, source.PenaltyTime, source.PlayerName)
                            .Controls.Add(ctrl)
                        Case 1
                            ctrl = ctrls(0)
                            With ctrl
                                .PenaltyTime = source.PenaltyTime
                                .PlayerNumber = source.PlayerNumber
                                .PlayerName = source.PlayerName
                            End With
                    End Select
                End With
                HomePenaltyPanel3.Controls.Clear()
            Case Else
                With HomePenaltyPanel1
                    Dim ctrl As PenaltyControl
                    Dim source As Penalty = CType(_ownerForm, Bedienung).PenaltyHomeBS.List(0)
                    Dim ctrls = .Controls
                    Select Case ctrls.Count
                        Case 0
                            ctrl = New PenaltyControl(source.PlayerNumber, source.PenaltyTime, source.PlayerName)
                            .Controls.Add(ctrl)
                        Case 1
                            ctrl = ctrls(0)
                            With ctrl
                                .PenaltyTime = source.PenaltyTime
                                .PlayerNumber = source.PlayerNumber
                                .PlayerName = source.PlayerName
                            End With
                    End Select
                End With
                With HomePenaltyPanel2
                    Dim ctrl As PenaltyControl
                    Dim source As Penalty = CType(_ownerForm, Bedienung).PenaltyHomeBS.List(1)
                    Dim ctrls = .Controls
                    Select Case ctrls.Count
                        Case 0
                            ctrl = New PenaltyControl(source.PlayerNumber, source.PenaltyTime, source.PlayerName)
                            .Controls.Add(ctrl)
                        Case 1
                            ctrl = ctrls(0)
                            With ctrl
                                .PenaltyTime = source.PenaltyTime
                                .PlayerNumber = source.PlayerNumber
                                .PlayerName = source.PlayerName
                            End With
                    End Select
                End With
                With HomePenaltyPanel3
                    Dim ctrl As PenaltyControl
                    Dim source As Penalty = CType(_ownerForm, Bedienung).PenaltyHomeBS.List(2)
                    Dim ctrls = .Controls
                    Select Case ctrls.Count
                        Case 0
                            ctrl = New PenaltyControl(source.PlayerNumber, source.PenaltyTime, source.PlayerName)
                            .Controls.Add(ctrl)
                        Case 1
                            ctrl = ctrls(0)
                            With ctrl
                                .PenaltyTime = source.PenaltyTime
                                .PlayerNumber = source.PlayerNumber
                                .PlayerName = source.PlayerName
                            End With
                    End Select
                End With
        End Select

        ' Change Panalty UI for Guest Team
        Select Case guestPenalties
            Case 0
                GuestPenaltyPanel1.Controls.Clear()
                GuestPenaltyPanel2.Controls.Clear()
                GuestPenaltyPanel3.Controls.Clear()
            Case < 2
                With GuestPenaltyPanel1
                    Dim ctrl As PenaltyControl
                    Dim source As Penalty = CType(_ownerForm, Bedienung).PenaltyGuestBS.List(0)
                    Dim ctrls = .Controls
                    Select Case ctrls.Count
                        Case 0
                            ctrl = New PenaltyControl(source.PlayerNumber, source.PenaltyTime, source.PlayerName)
                            .Controls.Add(ctrl)
                        Case 1
                            ctrl = ctrls(0)
                            With ctrl
                                .PenaltyTime = source.PenaltyTime
                                .PlayerNumber = source.PlayerNumber
                                .PlayerName = source.PlayerName
                            End With
                    End Select
                End With
                GuestPenaltyPanel2.Controls.Clear()
            Case < 3
                With GuestPenaltyPanel1
                    Dim ctrl As PenaltyControl
                    Dim source As Penalty = CType(_ownerForm, Bedienung).PenaltyGuestBS.List(0)
                    Dim ctrls = .Controls
                    Select Case ctrls.Count
                        Case 0
                            ctrl = New PenaltyControl(source.PlayerNumber, source.PenaltyTime, source.PlayerName)
                            .Controls.Add(ctrl)
                        Case 1
                            ctrl = ctrls(0)
                            With ctrl
                                .PenaltyTime = source.PenaltyTime
                                .PlayerNumber = source.PlayerNumber
                                .PlayerName = source.PlayerName
                            End With
                    End Select
                End With
                With GuestPenaltyPanel2
                    Dim ctrl As PenaltyControl
                    Dim source As Penalty = CType(_ownerForm, Bedienung).PenaltyGuestBS.List(1)
                    Dim ctrls = .Controls
                    Select Case ctrls.Count
                        Case 0
                            ctrl = New PenaltyControl(source.PlayerNumber, source.PenaltyTime, source.PlayerName)
                            .Controls.Add(ctrl)
                        Case 1
                            ctrl = ctrls(0)
                            With ctrl
                                .PenaltyTime = source.PenaltyTime
                                .PlayerNumber = source.PlayerNumber
                                .PlayerName = source.PlayerName
                            End With
                    End Select
                End With
                GuestPenaltyPanel3.Controls.Clear()
            Case Else
                With GuestPenaltyPanel1
                    Dim ctrl As PenaltyControl
                    Dim source As Penalty = CType(_ownerForm, Bedienung).PenaltyGuestBS.List(0)
                    Dim ctrls = .Controls
                    Select Case ctrls.Count
                        Case 0
                            ctrl = New PenaltyControl(source.PlayerNumber, source.PenaltyTime, source.PlayerName)
                            .Controls.Add(ctrl)
                        Case 1
                            ctrl = ctrls(0)
                            With ctrl
                                .PenaltyTime = source.PenaltyTime
                                .PlayerNumber = source.PlayerNumber
                                .PlayerName = source.PlayerName
                            End With
                    End Select
                End With
                With GuestPenaltyPanel2
                    Dim ctrl As PenaltyControl
                    Dim source As Penalty = CType(_ownerForm, Bedienung).PenaltyGuestBS.List(1)
                    Dim ctrls = .Controls
                    Select Case ctrls.Count
                        Case 0
                            ctrl = New PenaltyControl(source.PlayerNumber, source.PenaltyTime, source.PlayerName)
                            .Controls.Add(ctrl)
                        Case 1
                            ctrl = ctrls(0)
                            With ctrl
                                .PenaltyTime = source.PenaltyTime
                                .PlayerNumber = source.PlayerNumber
                                .PlayerName = source.PlayerName
                            End With
                    End Select
                End With
                With GuestPenaltyPanel3
                    Dim ctrl As PenaltyControl
                    Dim source As Penalty = CType(_ownerForm, Bedienung).PenaltyGuestBS.List(2)
                    Dim ctrls = .Controls
                    Select Case ctrls.Count
                        Case 0
                            ctrl = New PenaltyControl(source.PlayerNumber, source.PenaltyTime, source.PlayerName)
                            .Controls.Add(ctrl)
                        Case 1
                            ctrl = ctrls(0)
                            With ctrl
                                .PenaltyTime = source.PenaltyTime
                                .PlayerNumber = source.PlayerNumber
                                .PlayerName = source.PlayerName
                            End With
                    End Select
                End With
        End Select
    End Sub

    Public Sub ChangePeriodUI()
        Me.HalftimePB.Image = My.Resources.ResourceManager.GetObject("d_" & currentPeriod & "_blue")
    End Sub

    Private Sub ChangePointsUI(SelectedTeam As Integer, points As Integer) Handles Me.OnSavePointsComplete
        Dim pointLeftDigit As String = "0"
        Dim pointRightDigit As String = "0"
        If points < 10 Then
            pointRightDigit = points.ToString
        Else
            pointLeftDigit = points.ToString.Substring(0, 1)
            pointRightDigit = points.ToString.Substring(1, 1)
        End If

        Select Case SelectedTeam
            Case Team.Home
                With HomeLeftScorePB
                    .Image = My.Resources.ResourceManager.GetObject("d_" & pointLeftDigit & "_orange")
                End With
                With HomeRightScorePB
                    .Image = My.Resources.ResourceManager.GetObject("d_" & pointRightDigit & "_orange")
                End With
            Case Team.Guest
                With GuestLeftScorePB
                    .Image = My.Resources.ResourceManager.GetObject("d_" & pointLeftDigit & "_orange")
                End With
                With GuestRightScorePB
                    .Image = My.Resources.ResourceManager.GetObject("d_" & pointRightDigit & "_orange")
                End With
        End Select
    End Sub

    Private Sub OnGameTimerEnd(ByVal isHalf As Boolean)
        Dim dgResult As DialogResult
        Select Case isHalf
            Case True
                My.Computer.Audio.Play(My.Resources.BUZZER, AudioPlayMode.Background)
                dgResult = MessageBox.Show(GlobalStrings.NewHalfTimeMessage, GlobalStrings.MatchTimeHalf, MessageBoxButtons.OK)
                If (dgResult = DialogResult.OK) Then
                    currentPeriod += 1
                    ChangePeriodUI()
                End If
            Case False
                My.Computer.Audio.Play(My.Resources.BUZZER, AudioPlayMode.Background)
                If My.Settings.cupMatch = True Then
                    If _extraTimeRound < My.Settings.extraTimeRounds Then
                        dgResult = MessageBox.Show(GlobalStrings.CupMatchMessage, GlobalStrings.CupMatch, MessageBoxButtons.YesNo)
                        If dgResult = DialogResult.Yes Then
                            _extraTimeRound += 1
                            currentPeriod += 1
                            MsgBox(String.Format("Yes we want extra time for {0} rounds", _extraTimeRound))
                            ChangePeriodUI()

                        End If
                    End If
                Else

                End If
        End Select
    End Sub

    Private Sub OnTeamTimeout(selectedTeam As Integer)
        Select Case selectedTeam
            Case Team.Home
                HomeTTOPB.Image = My.Resources.ResourceManager.GetObject("to_green")
            Case Team.Guest
                GuestTTOPB.Image = My.Resources.ResourceManager.GetObject("to_green")
        End Select
        With Me.TimeOutTimer
            .Enabled = True
        End With
        ChangeTimeoutTimerUI()
    End Sub

    Private Sub TimeOutTimer_Tick(sender As Object, e As EventArgs) Handles TimeOutTimer.Tick
        Me.currentTimeout += 1000
        If (My.Settings.timeoutTime = currentTimeout) Then
            Me.TimeOutTimer.Stop()
            RaiseEvent OnTimeoutCompleted()
        ElseIf currentTimeout = 50000 Then
            My.Computer.Audio.Play(My.Resources.BUZZER, AudioPlayMode.Background)
            ChangeTimeoutTimerUI()
        Else
            ChangeTimeoutTimerUI()
        End If
    End Sub

    Private Sub ChangeTimeoutTimerUI()
        Dim Minutes As Integer = 0
        Dim Seconds As Integer = (My.Settings.timeoutTime - currentTimeout) / 1000
        If (Seconds > 59) Then
            Dim _convTime2Minutes = Seconds / 60
            Minutes = Convert.ToInt16(_convTime2Minutes.ToString.Split(",")(0))
            Seconds -= (Minutes * 60)
        End If

        Select Case Seconds
            Case < 10
                With Me.GameTimeSecondRightPB
                    .Image = My.Resources.ResourceManager.GetObject("d_" & Seconds.ToString & "_red")
                End With
                With Me.GameTimeSecondLeftPB
                    .Image = My.Resources.ResourceManager.GetObject("d_0_red")
                End With
            Case 10 To 59
                Dim digitLeft = Seconds.ToString.Substring(0, 1)
                Dim digitRight = Seconds.ToString.Substring(1, 1)
                With Me.GameTimeSecondRightPB
                    .Image = My.Resources.ResourceManager.GetObject("d_" & digitRight & "_red")
                End With
                With Me.GameTimeSecondLeftPB
                    .Image = My.Resources.ResourceManager.GetObject("d_" & digitLeft & "_red")
                End With
        End Select
        Select Case Minutes
            Case < 10
                With Me.GameTimeMinuteRightPB
                    .Image = My.Resources.ResourceManager.GetObject("d_" & Minutes.ToString & "_red")
                End With
                With Me.GameTimeMinuteLeftPB
                    .Image = My.Resources.ResourceManager.GetObject("d_0_red")
                End With
            Case 10 To 60
                Dim digitLeft = Minutes.ToString.Substring(0, 1)
                Dim digitRight = Minutes.ToString.Substring(1, 1)
                With Me.GameTimeMinuteRightPB
                    .Image = My.Resources.ResourceManager.GetObject("d_" & digitRight & "_red")
                End With
                With Me.GameTimeMinuteLeftPB
                    .Image = My.Resources.ResourceManager.GetObject("d_" & digitLeft & "_red")
                End With
            Case Else
                'Throw New IndexOutOfRangeException("Display could show only 60 Minutes or less")
        End Select
    End Sub

    Private Sub ChangeDisplayTeamName(name As String, seLTeam As Integer)
        Select Case seLTeam
            Case Team.Home
                HomeTeamLbl.Text = name
            Case Team.Guest
                GuestTeamLbl.Text = name
        End Select
    End Sub

    Private Sub ChangeGameTitle(content As String)
        GameTitleLbl.Text = content
    End Sub

    Private Sub ChangeInfoText(content As String)
        InfoTextLbl.Text = content
    End Sub

    Private Sub Timeout_Completed() Handles Me.OnTimeoutCompleted
        My.Computer.Audio.Play(My.Resources.BUZZER, AudioPlayMode.Background)
        currentTimeout = 0
        HomeTTOPB.Image = Nothing
        GuestTTOPB.Image = Nothing
        ActivateGamePanelControls()
        ChangeTimerUI(currentTime)
    End Sub

    Private Sub ActivateGamePanelControls()
        Me._ownerForm.GamePanel().StartStopBtn.Enabled = True
        Me._ownerForm.GamePanel().AddTTOGuestBtn.Enabled = True
        Me._ownerForm.GamePanel().AddTTOHomeBtn.Enabled = True
    End Sub

    Public Sub ResetGameData()
        currentPeriod = 1
        currentTime = 0
        currentTimeout = 0
        _homePoints = 0
        _guestPoints = 0
        Dim homePenalties As BindingSource = CType(_ownerForm, Bedienung).PenaltyHomeBS
        Dim guestPenalties As BindingSource = CType(_ownerForm, Bedienung).PenaltyGuestBS
        homePenalties.Clear()
        guestPenalties.Clear()
        Me.ChangePointsUI(Team.Home, _homePoints)
        Me.ChangePointsUI(Team.Guest, _guestPoints)
        Me.ChangePeriodUI()
        Me.ChangePenaltyUI()
        Me.ChangeTimerUI(currentTime)
    End Sub
End Class