﻿Imports System.IO
Imports System.Net
Imports System.Net.Sockets
Imports Microsoft.Win32
Imports Newtonsoft.Json
Imports System.Text
Imports Scoreboard.My.Resources
Imports System.Threading

Public Class Bedienung
    Dim _secondScreenAvailable As Boolean = False
    Dim _secondScreen As Screen = Nothing
    Dim _primaryScreen As Screen = Nothing
    Dim _secondBounds As Rectangle = Nothing
    Dim _videoForm As Anzeige = Nothing
    Dim _gamePanel As GameControl = Nothing
    Dim _retryVideoScreens As Integer = 0

    Dim _tbHasFocus As Boolean = False

    Private _oReg As RegistryKey

    ReadOnly _stream As NetworkStream
    ReadOnly _streamw As StreamWriter
    ReadOnly _streamr As StreamReader
    Dim _Server As TcpListener
    Dim _client As New TcpClient
    ReadOnly _ipendpoint As New IPEndPoint(IPAddress.Any, 3420)
    Dim _serverThread As Threading.Thread

    Dim IPAddr As String = ""

    Public ReadOnly Property TextBoxHasFocus() As Boolean
        Get
            Return _tbHasFocus
        End Get
    End Property

    Public WriteOnly Property SetFocusOnTextBox()
        Set(value)
            _tbHasFocus = value
        End Set
    End Property

    Public Event OnStopGameTimer()
    Public Event OnChangeVideoDisplayVisiblility()
    Public Event OnAddPenalty2BindingSourceComplete()
    Public Event OnChangePlayer2BindingSourceComplete(Status As Player.ChangeStatus)
    Public Event OnChangeTeamName(name As String, team As Integer)
    Public Event OnChangeGameTitle(content As String)
    Public Event OnChangeInfoText(content As String)
    Public Event OnSpacePressed()
    Public Event OnNumPlusPressed(team As Integer)
    Public Event OnNumMinusPressed(team As Integer)
    Public Event OnPlusPressed(team As Integer)
    Public Event OnMinusPressed(team As Integer)
    Public Event OnResetMatch()
    Public Event OnEditPenaltyComplete()

    Private Sub Bedienung_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        checkLanguage()
        _gamePanel = New GameControl()
        Me.CheckSecondScreen()
        Dim args() As String = Environment.GetCommandLineArgs()
        If args.Length > 1 Then
            Try
                Dim _confFile As String = args(1)
                Dim _str As String = File.ReadAllText(_confFile)
                Dim _conf As Configuration = JsonConvert.DeserializeObject(Of Configuration)(_str)
                MsgBox(String.Format("{0} wurde erfolgreich eingelesen.", _conf.Name), MsgBoxStyle.Information)
                OpenSettingsDialog(_conf)
            Catch ex As Exception
                MsgBox("Es wurde eine ungültige Konfiguration geladen", MsgBoxStyle.Exclamation)
            End Try
        Else
            InitRegistryKeys()
            If My.Settings.runSettingsScreenonStart Then
                OpenSettingsDialog()
            End If
        End If
        IPAddr = GetLocaleIpAddress()
        _serverThread = New Threading.Thread(AddressOf ScoreboardServer)
        InfoStatusLbl.Text = GetLocaleIpAddress()
        _serverThread.Start()
        NetworkTimer.Enabled = True
        AddHandler _gamePanel.OnRaiseTeamPenalty, AddressOf OnAddPenalty
        AddHandler _videoForm.OnSavePointsComplete, AddressOf OnChangeScore
        AddHandler _gamePanel.OnRaiseTeamPenalty, AddressOf OnAddPenaltyComplete
        AddHandler _gamePanel.OnRaiseTimeout, AddressOf OnAddTimeOut
        AddHandler OnChangeVideoDisplayVisiblility, AddressOf OnChangeVideoDisplay
        AddHandler OnResetMatch, AddressOf _videoForm.ResetGameData
        AddHandler _videoForm.OnResetMatch, AddressOf _gamePanel.RefreshGameTime
        AddHandler _videoForm.OnResetMatch, AddressOf ResetLabelText
        AddHandler _videoForm.OnGameTimerTick, AddressOf OnGameTimerTick
        AddHandler _videoForm.OnTimerEnd, AddressOf OnGameTimerEnd
        RaiseEvent OnChangeVideoDisplayVisiblility()
        Me.KeyPreview = True
    End Sub

    Private Sub NetworkTimer_Tick(sender As Object, e As EventArgs) Handles NetworkTimer.Tick
        NetworkTimer.Interval = 1000
        Using s As New Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp)
            Dim BCSTAddr As String = ""
            Dim IPAddrSplit() As String = IPAddr.Split("."c)
            BCSTAddr = IPAddrSplit(0) & "." & IPAddrSplit(1) & "." & IPAddrSplit(2) & ".255"
            Dim broadcast As IPAddress = IPAddress.Parse(BCSTAddr)
            Dim sendbuf As Byte() = Encoding.ASCII.GetBytes(IPAddr & "|Server = Scoreboard|LogicalPort = 3420|" & Now.ToLongTimeString)
            Dim ep As New IPEndPoint(broadcast, 11000)
            s.SendTo(sendbuf, ep)
        End Using

    End Sub

    Private Sub BeendenMenuItem_Click(sender As Object, e As EventArgs) Handles BeendenMenuItem.Click
        Me.Close()
    End Sub

    Private Sub CheckSecondScreen()
        _secondScreenAvailable = Screen.AllScreens.Count > 1
        If _secondScreenAvailable Then
            _secondScreen = Screen.AllScreens(1)
            If _secondScreen.Primary Then
                _primaryScreen = Screen.AllScreens(1)
                _secondScreen = Screen.AllScreens(0)
                Me.Location = Screen.AllScreens(1).WorkingArea.Location
            Else
                _primaryScreen = Screen.AllScreens(0)
                Me.Location = Screen.AllScreens(0).WorkingArea.Location
            End If
            _secondBounds = _secondScreen.Bounds
            With secondScreenLbl
                .Text = "Videoanzeige gefunden"
                .ForeColor = Color.Green
            End With
            Me.BuildVideoForm()
        Else
            Dim _result As DialogResult = MsgBox("Es konnte keine 2. Anzeige für das Scoreboard gefunden werden. Schliessen Sie die Videoanzeige via HDMI/DisplayPort/VGA an oder beenden Sie das Programm.", MsgBoxStyle.RetryCancel, "Auswahl Videoanzeige")
            If _result = DialogResult.Retry Then
                _retryVideoScreens += 1
                If _retryVideoScreens = 3 Then
                    Me.Close()
                Else
                    CheckSecondScreen()
                End If
            ElseIf _result = DialogResult.Cancel Then
                Me.Close()
            Else
                MsgBox("Fehler beim Abfragen der Videoanzeige. Programm wird beendet.", MsgBoxStyle.Exclamation, "Videoanzeige")
                Me.Close()
            End If
        End If
    End Sub

    Private Sub BuildVideoForm()
        If (_secondScreenAvailable) Then
            _videoForm = New Anzeige()
            AddHandler CType(_videoForm, Anzeige).OnGameTimerTick, AddressOf RefreshBindingSource
            With _videoForm
                .Owner = Me
                .BackColor = Color.Black
                .StartPosition = FormStartPosition.Manual
                .SetBounds(_secondBounds.X, _secondBounds.Y, _secondBounds.Width, _secondBounds.Height)
                .Show()
            End With
        End If
    End Sub

    Private Sub NewGameTSMI_Click(sender As Object, e As EventArgs) Handles NewGameTSMI.Click
        Dim controls As Control() = SplitContainer1.Panel2.Controls.Find("GameControl", True)
        If controls.Count > 0 Then
            Dim dgResult As DialogResult = MessageBox.Show(GlobalStrings.CancelGameMessage, GlobalStrings.CancelGame, MessageBoxButtons.YesNo)
            If dgResult = DialogResult.Yes Then
                SplitContainer1.Panel2.Controls.Remove(_gamePanel)
                Dim settingsResult = OpenSettingsDialog()
                If settingsResult = DialogResult.OK Then
                    GamelogBS.Clear()
                    ScoreLbl.Text = "0:0"
                    TimeLbl.Text = "00:00"
                    RaiseEvent OnResetMatch()
                End If
            End If
        End If
        _gamePanel.Dock = DockStyle.Fill
        SplitContainer2.Panel2.Controls.Add(_gamePanel)
    End Sub

    Private Sub OnChangeVideoDisplay()
        If Not _secondScreenAvailable Then
            Exit Sub
        End If
    End Sub

    Public Function GamePanel() As GameControl
        Return _gamePanel
    End Function

    Public Function VideoPanel() As Anzeige
        Return _videoForm
    End Function

    Private Sub OnAddPenalty(Sender As Object, e As PenaltyEventArgs)
        Dim newPenalty As New Penalty(CType(_videoForm, Anzeige).GetCurrentGameTime(), e.PenaltyTeam, e.PlayeNumber, e.PlayerName, e.PenaltyTime)
        Select Case e.PenaltyTeam
            Case Team.Home
                With PenaltyHomeBS
                    .Add(newPenalty)
                End With
            Case Team.Guest
                With PenaltyGuestBS
                    .Add(newPenalty)
                End With
        End Select
        Dim gl As New GameLog(CType(_videoForm, Anzeige).GetCurrentGameTime(), newPenalty, CType(_videoForm, Anzeige).HomePoints, CType(_videoForm, Anzeige).GuestPoints)
        With GamelogBS
            .Add(gl)
        End With

        Me.GamelogDGV.FirstDisplayedScrollingRowIndex = Me.GamelogDGV.RowCount - 1
        RaiseEvent OnAddPenalty2BindingSourceComplete()
    End Sub

    Private Sub OnAddTimeOut(who As Team)
        Dim teamName As String = Nothing
        Select Case who
            Case Team.Home
                teamName = GlobalStrings.Home
            Case Team.Guest
                teamName = GlobalStrings.Guest
        End Select
        Dim gl As New GameLog(String.Format("Timeout for {0}", teamName), VideoPanel().GetCurrentGameTime, who)
        With GamelogBS
            .Add(gl)
        End With
        Me.GamelogDGV.FirstDisplayedScrollingRowIndex = Me.GamelogDGV.RowCount - 1
    End Sub

    Private Sub RefreshBindingSource()
        Dim HomeList2Remove As New List(Of Penalty)
        Dim GuestList2Remove As New List(Of Penalty)
        Dim currentTime As Integer = CType(_videoForm, Anzeige).GetCurrentGameTime()
        For Each source As Penalty In PenaltyHomeBS.List
            source.PenaltyTime -= 1000
            If source.PenaltyTime <= 0 Then
                source.DeletePenalty = True
                HomeList2Remove.Add(source)
            End If
        Next

        For Each source2Delete As Penalty In HomeList2Remove
            PenaltyHomeBS.Remove(source2Delete)
        Next
        PenaltyHomeBS.ResetBindings(True)

        For Each source As Penalty In PenaltyGuestBS.List
            source.PenaltyTime -= 1000
            If source.PenaltyTime <= 0 Then
                source.DeletePenalty = True
                GuestList2Remove.Add(source)
            End If
        Next

        For Each source2Delete As Penalty In GuestList2Remove
            PenaltyGuestBS.Remove(source2Delete)
        Next
        PenaltyGuestBS.ResetBindings(True)
        RaiseEvent OnAddPenalty2BindingSourceComplete()
    End Sub

    Private Sub DGV_CellFormatting(sender As Object, e As DataGridViewCellFormattingEventArgs) Handles HomePenaltyDGV.CellFormatting, GuestPenaltyDGV.CellFormatting
        If e.ColumnIndex = DirectCast(sender, DataGridView).Columns(6).Index Then
            e.Value = String.Format(New PenaltyTimeFormatProvider(), "{0:U}", e.Value)
            e.FormattingApplied = True
        End If
    End Sub

    Private Sub GameLog_Cellformatting(sender As Object, e As DataGridViewCellFormattingEventArgs) Handles GamelogDGV.CellFormatting
        If e.ColumnIndex = GamelogDGV.Columns("getGameTime").Index Then
            e.Value = String.Format(New PenaltyTimeFormatProvider(), "{0:U}", e.Value)
            e.FormattingApplied = True
        End If
    End Sub

    Private Function OpenSettingsDialog(Optional loadConfig As Configuration = Nothing)
        Dim settingsFrm As New Settings()
        If loadConfig IsNot Nothing Then
            settingsFrm.LoadConfiguration = loadConfig
        End If
        With settingsFrm
            .Owner = Me
            .Text = "Einstellungen"
            .SettingsTabControl.SelectedTab = .SettingsTabControl.TabPages(0)
            .Location = _primaryScreen.WorkingArea.Location
            .ShowIcon = True
            .Icon = My.Resources.handball
            .StartPosition = FormStartPosition.Manual
            .ShowDialog()
        End With

        Return settingsFrm.DialogResult
    End Function

    Private Sub SettingsTSMI_Click(sender As Object, e As EventArgs) Handles SettingsTSMI.Click
        OpenSettingsDialog()
    End Sub

    Private Sub TeamNameTB_TextChanged(sender As Object, e As EventArgs) Handles HomeTeamTb.TextChanged, GuestTeamTb.TextChanged
        Select Case CType(sender, TextBox).Name.ToLower
            Case "hometeamtb"
                RaiseEvent OnChangeTeamName(CType(sender, TextBox).Text, Team.Home)
            Case "guestteamtb"
                RaiseEvent OnChangeTeamName(CType(sender, TextBox).Text, Team.Guest)
        End Select
    End Sub

    Private Sub GameTitleTB_TextChanged(sender As Object, e As EventArgs) Handles GameTitleTB.TextChanged
        RaiseEvent OnChangeGameTitle(CType(sender, TextBox).Text)
    End Sub

    Private Sub Textbox_GotFocus(sender As Object, e As EventArgs) Handles GameTitleTB.GotFocus
        Console.WriteLine(CType(sender, TextBox).Name & " Got Focus")
        _tbHasFocus = True
    End Sub

    Private Sub Textbox_Leave(sender As Object, e As EventArgs) Handles GameTitleTB.Leave
        Console.WriteLine(CType(sender, TextBox).Name & " lost Focus")
        _tbHasFocus = False
    End Sub

    Private Sub InfoTextTB_TextChanged(sender As Object, e As EventArgs) Handles InfoTextTB.TextChanged
        RaiseEvent OnChangeInfoText(CType(sender, TextBox).Text)
    End Sub

    Private Sub CheckBox1_CheckedChanged(sender As Object, e As EventArgs) Handles CupMatchCB.CheckedChanged
        My.Settings.cupMatch = CType(sender, CheckBox).Checked
        My.Settings.Save()
    End Sub

    Private Sub Bedienung_FormClosed(sender As Object, e As EventArgs) Handles MyBase.FormClosing
        If _Server IsNot Nothing Then
            _Server.Stop()
        End If

        Try
            _serverThread.Abort()
        Catch
        End Try
    End Sub

    Sub ScoreboardServer()
        Try
            _Server = New TcpListener(_ipendpoint)
            _Server.Start()
            While True
                _client = _Server.AcceptTcpClient
                Dim c As New ClientConnection(
                    _client.GetStream,
                    New StreamWriter(_client.GetStream),
                    New StreamReader(_client.GetStream),
                    _client.Client.RemoteEndPoint) ' und erstellen für die neue verbindung eine neue connection...

                ClientConnectionBS.Add(c) ' und fügen sie der liste der clients hinzu.
                ClientConnectionBS.ResetBindings(False)
                Dim t As New Threading.Thread(AddressOf ListenToConnection)
                t.Start(c)
            End While
        Catch ex As Exception
        End Try
    End Sub

    Private Sub ListenToConnection(ByVal con As ClientConnection)
        Do
            Try
                Dim tmp As String = con.Reader.ReadLine ' warten, bis etwas empfangen wird...
                SendToAllClients(con.Nickname & ": " & tmp) ' an alle clients weitersenden.
            Catch ' die aktuelle überwachte verbindung hat sich wohl verabschiedet.
                ClientConnectionBS.Remove(con)
                ClientConnectionBS.ResetBindings(False)
                Exit Do
            End Try
        Loop
    End Sub
    Private Sub SendToAllClients(ByVal s As String)
        For Each c As ClientConnection In ClientConnectionBS.List  ' an alle clients weitersenden.
            Try
                c.Writer.WriteLine(s)
                c.Writer.Flush()
            Catch
            End Try
        Next
    End Sub

    Private Sub SendToClient(ByVal c As ClientConnection, ByVal data As String)
        c.Writer.WriteLine(data)
        c.Writer.Flush()
    End Sub

    Private Function GetLocaleIpAddress() As String
        Dim host = Dns.GetHostEntry(Dns.GetHostName)
        For Each _ip As IPAddress In host.AddressList
            If _ip.AddressFamily = AddressFamily.InterNetwork Then
                Return _ip.ToString
            End If
        Next
        Throw New Exception("No Network Adapters with an IPv4 address in the system!")
    End Function

    Private Sub OnChangeScore(ByVal who As Integer, points As Integer)
        SendToAllClients(String.Format("scoreboard|points:who|{0}:points|{1}:game_time|{2}", who, points, VideoPanel().GetCurrentGameTime))
        With ScoreLbl
            .Text = String.Format("{0}:{1}", Me.VideoPanel().HomePoints, Me.VideoPanel().GuestPoints)
        End With
        Dim gl As New GameLog(VideoPanel().GetCurrentGameTime, who, Me.VideoPanel().HomePoints, Me.VideoPanel().GuestPoints)
        With GamelogBS
            .Add(gl)
        End With
        Me.GamelogDGV.FirstDisplayedScrollingRowIndex = Me.GamelogDGV.RowCount - 1
    End Sub

    Private Sub OnAddPenaltyComplete(sender As Object, e As PenaltyEventArgs)
        If Not e.EditMode Then
            SendToAllClients(String.Format("scoreboard|penalty:who|{0}:player|{1}:penalty|{2}:game_time|{3}", e.PenaltyTeam, e.PlayeNumber, e.PenaltyTime, VideoPanel().GetCurrentGameTime))
        End If

    End Sub

    Private Sub OnGameTimerTick(currentTime As Integer)
        Dim Minutes As Integer = 0
        Dim Seconds As Integer = currentTime / 1000
        If (Seconds > 59) Then
            Dim _convTime2Minutes = Seconds / 60
            Minutes = Convert.ToInt16(_convTime2Minutes.ToString.Split(",")(0))
            Seconds -= (Minutes * 60)
        End If
        With TimeLbl
            .Text = String.Format("{0}:{1}", Minutes.ToString("0#"), Seconds.ToString("0#"))
        End With
    End Sub

    Private Sub OnGameTimerEnd(isHalf As Boolean)
        Dim gl As New GameLog(If(isHalf = True, "Halbzeit", "Spielende"), Me.VideoPanel().GetCurrentGameTime, Me.VideoPanel().HomePoints, Me.VideoPanel().GuestPoints, Me.VideoPanel().GetPeriod)
        With GamelogBS
            .Add(gl)
        End With
        Me.GamelogDGV.FirstDisplayedScrollingRowIndex = Me.GamelogDGV.RowCount - 1
    End Sub

    Private Sub OnTextBox_Enter(sender As Object, e As EventArgs) Handles GameTitleTB.Enter, InfoTextTB.Enter, HomeTeamTb.Enter, GuestTeamTb.Enter
        SetFocusOnTextBox = True
    End Sub

    Private Sub OnTextBox_Leave(sender As Object, e As EventArgs) Handles GameTitleTB.Leave, InfoTextTB.Leave, HomeTeamTb.Leave, GuestTeamTb.Leave
        SetFocusOnTextBox = False
    End Sub

    Private Sub InitRegistryKeys()
        Dim appExeName As String = System.IO.Path.GetFileName(Application.ExecutablePath)
        _oReg = Registry.CurrentUser.OpenSubKey("Software\\Classes\Applications\\" & appExeName, True)
        If _oReg Is Nothing Then
            Exit Sub
        End If

        Dim _settingsReg As RegistryKey = _oReg.OpenSubKey("Settings", True)
        If _settingsReg Is Nothing Then
            Exit Sub
        End If

        My.Settings.gameTime = ConvertTimeFromString(_settingsReg.GetValue("gameTime"))
        My.Settings.halfTime = ConvertTimeFromString(_settingsReg.GetValue("halfTime"))
        My.Settings.timeoutTime = ConvertTimeFromString(_settingsReg.GetValue("timeoutTime"))
        My.Settings.gamePeriods = ConvertIntFromString(_settingsReg.GetValue("gamePeriods"))
        My.Settings.cupMatch = ConvertBoolFromString(_settingsReg.GetValue("cupMatch"))
        My.Settings.extraTime = ConvertTimeFromString(_settingsReg.GetValue("extraTime"))
        My.Settings.extraHalftime = ConvertTimeFromString(_settingsReg.GetValue("extraHalftime"))
        My.Settings.extraTimeRounds = ConvertIntFromString(_settingsReg.GetValue("extraTimeRounds"))
        My.Settings.Save()
    End Sub

    Private Function ConvertTimeFromString(content As String) As Integer
        Dim newTimeStr As String = content
        Dim _minutes As Integer
        Dim _seconds As Integer
        If newTimeStr.Length = 0 Then
            Return 0
        ElseIf newTimeStr.IndexOf(":") = -1 And newTimeStr.Length <= 2 Then
            _minutes = Convert.ToInt16(newTimeStr)
        Else
            _minutes = Convert.ToInt16(newTimeStr.Substring(0, 2))
            _seconds = Convert.ToInt16(newTimeStr.Substring(3, 2))
        End If
        Return ((_minutes * 60) + _seconds) * 1000
    End Function

    Private Function ConvertIntFromString(content As String) As Integer
        If content.Length < 1 Then
            Return 0
        Else
            Return CInt(content)
        End If
    End Function

    Private Function ConvertBoolFromString(content As String) As Boolean
        If content.Length < 1 Then
            Return False
        Else
            Select Case content
                Case "0"
                    Return False
                Case "1"
                    Return True
                Case "false"
                    Return False
                Case "true"
                    Return True
                Case Else
                    Return False
            End Select
        End If
    End Function


    Public Sub ResetLabelText()
        With TimeLbl
            .Text = "00:00"
        End With
    End Sub
    Dim clickedCell As DataGridViewCell
    Private Sub PenaltyDGV_CellMouseDown(sender As Object, e As DataGridViewCellMouseEventArgs) Handles HomePenaltyDGV.CellMouseDown, GuestPenaltyDGV.CellMouseDown
        If e.RowIndex > -1 And e.ColumnIndex > -1 Then
            If e.Button = MouseButtons.Right Then
                clickedCell = CType(sender, DataGridView).Rows(e.RowIndex).Cells(e.ColumnIndex)
                Dim relMousePos = CType(sender, DataGridView).PointToClient(Cursor.Position)
                PenaltyContextMenuStrip.Show(sender, relMousePos)
            Else
                clickedCell = Nothing
            End If
        Else
            clickedCell = Nothing
        End If
        CType(sender, DataGridView).CurrentCell = clickedCell
        Select Case CType(sender, DataGridView).Name
            Case "HomePenaltyDGV"
                GuestPenaltyDGV.CurrentCell = Nothing
            Case "GuestPenaltyDGV"
                HomePenaltyDGV.CurrentCell = Nothing
        End Select
    End Sub

    Private Sub PlayerDGV_CellMouseDown(sender As Object, e As DataGridViewCellMouseEventArgs) Handles HomePlayerListDGV.CellMouseDown, GuestPlayerListDGV.CellMouseDown
        If e.RowIndex > -1 And e.ColumnIndex > -1 Then
            If e.Button = MouseButtons.Right Then
                clickedCell = CType(sender, DataGridView).Rows(e.RowIndex).Cells(e.ColumnIndex)
                Dim relMousePos = CType(sender, DataGridView).PointToClient(Cursor.Position)
                PlayerContextMenuStrip.Show(sender, relMousePos)
            Else
                clickedCell = Nothing
            End If
        Else
            clickedCell = Nothing
        End If
        CType(sender, DataGridView).CurrentCell = clickedCell
        Select Case CType(sender, DataGridView).Name
            Case "HomePlayerListDGV"
                GuestPlayerListDGV.CurrentCell = Nothing
            Case "GuestPlayerListDGV"
                HomePlayerListDGV.CurrentCell = Nothing
        End Select
    End Sub

    Private Sub EditPenaltyTSMI_Click(sender As Object, e As EventArgs) Handles EditPenaltyTSMI.Click
        Dim row As Penalty = Nothing
        If clickedCell IsNot Nothing Then
            If clickedCell.DataGridView IsNot Nothing Then
                row = CType(clickedCell.DataGridView.Rows(clickedCell.RowIndex).DataBoundItem, Penalty)
            End If
        End If

        If row Is Nothing Then
            Return
        End If

        Select Case row.PlayerTeam
            Case Team.Home
                For Each source As Penalty In PenaltyHomeBS.List
                    If source.ID = row.ID Then
                        Dim _dialogWindow As New Form() With {
                             .MaximizeBox = False,
                            .MinimizeBox = False,
                            .FormBorderStyle = FormBorderStyle.FixedDialog,
                            .AutoSize = True,
                            .Text = String.Format("Edit Penalty for Player {0} ({1}) - Home Team", source.PlayerName, source.PlayerNumber),
                            .StartPosition = FormStartPosition.CenterParent
                        }
                        Dim _dialogCtrl As New SelectTeamDialog(source.PlayerTeam)
                        _dialogCtrl.PlayerNameTB.Text = source.PlayerName
                        _dialogCtrl.PlayerNumberTB.Value = source.PlayerNumber
                        _dialogCtrl.PenaltyTimeTB.Text = GameLog.ConvertTimeToString(source.PenaltyTime)
                        _dialogWindow.Controls.Add(_dialogCtrl)
                        _dialogWindow.Size = _dialogCtrl.Size
                        Dim dlgResult As DialogResult = _dialogWindow.ShowDialog(Me)
                        If dlgResult = DialogResult.OK Then
                            With source
                                .PenaltyTime = ConvertTimeFromString(_dialogCtrl.PenaltyTimeTB.Text)
                                .PlayerName = _dialogCtrl.PlayerNameTB.Text
                                .PlayerNumber = _dialogCtrl.PlayerNumberTB.Value
                                .GameTime = _videoForm.GetCurrentGameTime
                            End With
                            PenaltyHomeBS.EndEdit()
                            PenaltyHomeBS.ResetBindings(False)
                            RaiseEvent OnEditPenaltyComplete()
                            Exit For
                        End If
                    End If
                Next
            Case Team.Guest
                For Each source As Penalty In PenaltyGuestBS.List
                    If source.ID = row.ID Then
                        Dim _dialogWindow As New Form() With {
                            .MaximizeBox = False,
                            .MinimizeBox = False,
                            .FormBorderStyle = FormBorderStyle.FixedDialog,
                            .AutoSize = True,
                            .Text = String.Format("Edit Penalty for Player {0} ({1}) - Guest Team", source.PlayerName, source.PlayerNumber),
                            .StartPosition = FormStartPosition.CenterParent
                        }

                        Dim _dialogCtrl As New SelectTeamDialog(source.PlayerTeam)
                        With _dialogCtrl
                            .PlayerNameTB.Text = source.PlayerName
                            .PlayerNumberTB.Value = source.PlayerNumber
                            .PenaltyTimeTB.Text = GameLog.ConvertTimeToString(source.PenaltyTime)
                        End With

                        _dialogWindow.Controls.Add(_dialogCtrl)
                        _dialogWindow.Size = _dialogCtrl.Size
                        Dim dlgResult As DialogResult = _dialogWindow.ShowDialog(Me)
                        If dlgResult = DialogResult.OK Then
                            With source
                                .PenaltyTime = ConvertTimeFromString(_dialogCtrl.PenaltyTimeTB.Text)
                                .PlayerName = _dialogCtrl.PlayerNameTB.Text
                                .PlayerNumber = _dialogCtrl.PlayerNumberTB.Value
                                .GameTime = _videoForm.GetCurrentGameTime
                            End With
                            PenaltyGuestBS.EndEdit()
                            PenaltyGuestBS.ResetBindings(False)
                            RaiseEvent OnEditPenaltyComplete()
                            Exit For
                        End If
                    End If
                Next
        End Select
    End Sub

    Private Sub EditPlayerTSMI_Click(sender As Object, e As EventArgs) Handles EditPlayerTSMI.Click
        Dim row As Player = Nothing
        If clickedCell IsNot Nothing Then
            If clickedCell.DataGridView IsNot Nothing Then
                row = CType(clickedCell.DataGridView.Rows(clickedCell.RowIndex).DataBoundItem, Player)
            End If
        End If

        If row Is Nothing Then
            Return
        End If

        Select Case row.PlayerTeam
            Case Team.Home
                For Each source As Player In HomePlayerListBS.List
                    If source.ID = row.ID Then
                        Dim _dialogWindow As New Form() With {
                             .MaximizeBox = False,
                            .MinimizeBox = False,
                            .FormBorderStyle = FormBorderStyle.FixedDialog,
                            .AutoSize = True,
                            .Text = String.Format("Edit Player {0} ({1}) - Home Team", source.PlayerName, source.PlayerNumber),
                            .StartPosition = FormStartPosition.CenterParent
                        }
                        Dim _dialogCtrl As New SelectPlayerDialog()
                        _dialogCtrl.PlayerNameTB.Text = source.PlayerName
                        If source.IsOfficial Then
                            _dialogCtrl.IsOfficialCB.Checked = True
                            _dialogCtrl.OfficialNumberDDL.SelectedIndex = _dialogCtrl.OfficialNumberDDL.FindStringExact(source.OfficlaNumber)
                        Else
                            _dialogCtrl.PlayerNumberTB.Value = source.PlayerNumber
                        End If

                        _dialogWindow.Controls.Add(_dialogCtrl)
                        _dialogWindow.Size = _dialogCtrl.Size
                        Dim dlgResult As DialogResult = _dialogWindow.ShowDialog(Me)
                        If dlgResult = DialogResult.OK Then
                            With source
                                .PlayerName = _dialogCtrl.PlayerNameTB.Text
                                .IsOfficial = _dialogCtrl.IsOfficialCB.Checked
                                If _dialogCtrl.IsOfficialCB.Checked Then
                                    .OfficlaNumber = _dialogCtrl.OfficialNumberDDL.SelectedItem.ToString()
                                    .PlayerNumber = Nothing
                                Else
                                    .PlayerNumber = _dialogCtrl.PlayerNumberTB.Value
                                End If
                            End With
                            HomePlayerListBS.EndEdit()
                            HomePlayerListBS.ResetBindings(False)
                            Exit For
                        End If
                    End If
                Next
            Case Team.Guest
                For Each source As Player In GuestPlayerListBS.List
                    If source.ID = row.ID Then
                        Dim _dialogWindow As New Form() With {
                             .MaximizeBox = False,
                            .MinimizeBox = False,
                            .FormBorderStyle = FormBorderStyle.FixedDialog,
                            .AutoSize = True,
                            .Text = String.Format("Edit Player {0} ({1}) - Guest Team", source.PlayerName, source.PlayerNumber),
                            .StartPosition = FormStartPosition.CenterParent
                        }
                        Dim _dialogCtrl As New SelectPlayerDialog()
                        _dialogCtrl.PlayerNameTB.Text = source.PlayerName
                        If source.IsOfficial Then
                            _dialogCtrl.IsOfficialCB.Checked = True
                            _dialogCtrl.OfficialNumberDDL.SelectedIndex = _dialogCtrl.OfficialNumberDDL.FindStringExact(source.OfficlaNumber)
                        Else
                            _dialogCtrl.PlayerNumberTB.Value = source.PlayerNumber
                        End If
                        _dialogWindow.Controls.Add(_dialogCtrl)
                        _dialogWindow.Size = _dialogCtrl.Size
                        Dim dlgResult As DialogResult = _dialogWindow.ShowDialog(Me)
                        If dlgResult = DialogResult.OK Then
                            With source
                                .PlayerName = _dialogCtrl.PlayerNameTB.Text

                                .IsOfficial = _dialogCtrl.IsOfficialCB.Checked
                                If _dialogCtrl.IsOfficialCB.Checked Then
                                    .OfficlaNumber = _dialogCtrl.OfficialNumberDDL.SelectedItem.ToString()
                                    .PlayerNumber = Nothing
                                Else
                                    .PlayerNumber = _dialogCtrl.PlayerNumberTB.Value
                                End If
                            End With
                            GuestPlayerListBS.EndEdit()
                            GuestPlayerListBS.ResetBindings(False)
                            Exit For
                        End If
                    End If
                Next
        End Select
    End Sub

    Private Sub RemovePenaltyTSMI_Click(sender As Object, e As EventArgs) Handles RemovePenaltyTSMI.Click
        Dim row As Penalty
        Dim HomeList2Remove As New List(Of Penalty)
        Dim GuestList2Remove As New List(Of Penalty)
        If HomePenaltyDGV.CurrentCell IsNot Nothing Then
            row = CType(HomePenaltyDGV.Rows(HomePenaltyDGV.CurrentCell.RowIndex).DataBoundItem, Penalty)
        ElseIf GuestPenaltyDGV.CurrentCell IsNot Nothing Then
            row = CType(GuestPenaltyDGV.Rows(GuestPenaltyDGV.CurrentCell.RowIndex).DataBoundItem, Penalty)
        Else
            Return
        End If

        If row Is Nothing Then
            Return
        End If

        Select Case row.PlayerTeam
            Case Team.Home
                For Each source As Penalty In PenaltyHomeBS.List
                    If source.ID = row.ID Then
                        source.DeletePenalty = True
                        HomeList2Remove.Add(source)
                    End If
                Next
            Case Team.Guest
                For Each source As Penalty In PenaltyGuestBS.List
                    If source.ID = row.ID Then
                        source.DeletePenalty = True
                        GuestList2Remove.Add(source)
                    End If
                Next
        End Select
        For Each source2Delete As Penalty In HomeList2Remove
            PenaltyHomeBS.Remove(source2Delete)
        Next
        PenaltyHomeBS.ResetBindings(True)

        For Each source2Delete As Penalty In GuestList2Remove
            PenaltyGuestBS.Remove(source2Delete)
        Next

        PenaltyGuestBS.ResetBindings(True)

        HomeList2Remove.Clear()
        GuestList2Remove.Clear()

        RaiseEvent OnAddPenalty2BindingSourceComplete()
    End Sub

    Private Sub PlayerRemoveTSMI_Click(sender As Object, e As EventArgs) Handles RemovePlayerTSMI.Click
        Dim row As Player
        Dim HomeList2Remove As New List(Of Player)
        Dim GuestList2Remove As New List(Of Player)

        If HomePlayerListDGV.CurrentCell IsNot Nothing Then
            row = CType(HomePlayerListDGV.Rows(HomePlayerListDGV.CurrentCell.RowIndex).DataBoundItem, Player)
        ElseIf GuestPlayerListDGV.CurrentCell IsNot Nothing Then
            row = CType(GuestPlayerListDGV.Rows(GuestPlayerListDGV.CurrentCell.RowIndex).DataBoundItem, Player)
        Else
            Exit Sub
        End If

        If row Is Nothing Then
            Exit Sub
        End If

        Select Case row.PlayerTeam
            Case Team.Home
                For Each source As Player In HomePlayerListBS.List
                    If source.ID = row.ID Then
                        source.DeletePlayer = True
                        HomeList2Remove.Add(source)
                    End If
                Next
            Case Team.Guest
                For Each source As Player In GuestPlayerListBS.List
                    If source.ID = row.ID Then
                        source.DeletePlayer = True
                        GuestList2Remove.Add(source)
                    End If
                Next
        End Select
        For Each source2Delete As Player In HomeList2Remove
            HomePlayerListBS.Remove(source2Delete)
        Next
        HomePlayerListBS.ResetBindings(True)

        For Each source2Delete As Player In GuestList2Remove
            GuestPlayerListBS.Remove(source2Delete)
        Next

        GuestPlayerListBS.ResetBindings(True)

        HomeList2Remove.Clear()
        GuestList2Remove.Clear()

        RaiseEvent OnChangePlayer2BindingSourceComplete(Player.ChangeStatus.RemovePlayer)
    End Sub

    Private Sub TSI_Language_DE_Click(sender As Object, e As EventArgs) Handles TSI_Language_DE.Click
        My.Settings.appLanguage = "de"
        My.Settings.Save()
        Dim dgResult As DialogResult = MsgBox(GlobalStrings.ChangeLanguageRestartAppMessage, MsgBoxStyle.YesNo, GlobalStrings.RestartApp)
        If dgResult = DialogResult.Yes Then
            Application.Restart()
        End If

        Application.Restart()
    End Sub

    Private Sub TSI_Language_GB_Click(sender As Object, e As EventArgs) Handles TSI_Language_GB.Click
        My.Settings.appLanguage = "en"
        My.Settings.Save()
        Dim dgResult As DialogResult = MsgBox(GlobalStrings.ChangeLanguageRestartAppMessage, MsgBoxStyle.YesNo, GlobalStrings.RestartApp)
        If dgResult = DialogResult.Yes Then
            Application.Restart()
        End If

    End Sub

    Private Sub CheckLanguage()
        If My.Settings.appLanguage = "de" Then
            TSI_Language_GB.Enabled = True
            TSI_Language_DE.Enabled = False
        ElseIf My.Settings.appLanguage = "en" Then
            TSI_Language_GB.Enabled = False
            TSI_Language_DE.Enabled = True
        End If
    End Sub

    Private Sub ProtocolExportBtn_Click(sender As Object, e As EventArgs) Handles ProtocolExportBtn.Click
        Me.ExportBW = New System.ComponentModel.BackgroundWorker() With {
            .WorkerReportsProgress = True,
            .WorkerSupportsCancellation = True
        }
        AddHandler Me.ExportBW.DoWork, AddressOf Bw_DoWOrk
        Dim protocoll As New PdfProtocoll(GameTitleTB.Text, "Game Log", "SV Sachsenring e.V.")
        protocoll.addGameData(GameTitleTB.Text, HomeTeamTb.Text, GuestTeamTb.Text, CupMatchCB.Checked)
        protocoll.AddEntries(GamelogBS)
        If HomePlayerListBS.Count > 0 Then
            protocoll.AddPLayerList(HomePlayerListBS, Team.Home, True)
        End If
        If GuestPlayerListBS.Count > 0 Then
            protocoll.AddPLayerList(GuestPlayerListBS, Team.Guest, True)
        End If
        Dim renderer As New MigraDoc.Rendering.PdfDocumentRenderer(True) With {
            .Document = protocoll.PdfDocument
        }
        renderer.RenderDocument()
        Dim logstream As Stream
        Dim savelogdialog As New SaveFileDialog() With {
            .Title = "",
            .Filter = "Log files (*.pdf)|*.pdf",
            .FileName = "gamelog",
            .InitialDirectory = Application.StartupPath,
            .FilterIndex = 2,
            .RestoreDirectory = False
        }

        If savelogdialog.ShowDialog() = DialogResult.OK Then
            Try
                logstream = savelogdialog.OpenFile()
                If logstream IsNot Nothing Then
                    BWPRogressBar.Visible = True
                    CType(sender, Button).Enabled = False
                    Me.ExportBW.RunWorkerAsync({logstream, renderer, protocoll})
                End If
            Catch ex As Exception
                MsgBox(GlobalStrings.ErrorExportMessage, MsgBoxStyle.Exclamation, GlobalStrings.ErrorExport)
            End Try

        End If
    End Sub

    Delegate Sub ChangeTextsSafe(ByVal length As Long, ByVal position As Integer, ByVal percent As Integer)
    Delegate Sub AddPlayerSafe(dataRow As Player, playerTeam As Team)
    Delegate Sub BackgroundWorkerCompleteSafe(cancelled As Boolean)
    Delegate Sub BackgroundWorkerImportCompleteSafe(cancelled As Boolean)

    Private Sub Bw_DoWOrk(sender As Object, e As System.ComponentModel.DoWorkEventArgs)
        Dim obj As Object = e.Argument
        Dim logstream As Stream = obj(0)
        obj(1).Save(logstream, False)
        Dim length As Long = logstream.Length
        Dim buffer(10) As Byte
        Dim nRead As Integer
        Dim safedelegate As New ChangeTextsSafe(AddressOf ChangeTexts)
        Me.Invoke(safedelegate, length, 0, 0)
        Do
            If ExportBW.CancellationPending Then
                Exit Do
            End If
            Dim bytesRead As Integer = logstream.Read(buffer, 0, 10)
            nRead += bytesRead
            Dim percent As Short = (nRead * 100) / length
            Me.Invoke(safedelegate, length, nRead, percent)
            If bytesRead = 0 Then Exit Do
        Loop
        logstream.Close()
        If ExportBW.CancellationPending Then
            Dim cancelDelegate As New BackgroundWorkerCompleteSafe(AddressOf DownloadComplete)
            Me.Invoke(cancelDelegate, True)
            Exit Sub
        End If
        Dim completeDelegate As New BackgroundWorkerCompleteSafe(AddressOf DownloadComplete)
        Me.Invoke(completeDelegate, False)

    End Sub

    Public Sub ChangeTexts(ByVal length As Long, ByVal position As Integer, ByVal percent As Integer)

        With Me.BWPRogressBar
            .Value = percent
        End With
    End Sub

    Public Sub DownloadComplete(ByVal cancelled As Boolean)
        With Me.BWPRogressBar
            .Visible = False
            .Value = 0
        End With
        ProtocolExportBtn.Enabled = True
        If cancelled Then
            MsgBox("Save Gamelog Process cancelled")
        End If
    End Sub

    Private Sub AddLogoBtn_Click(sender As Object, e As EventArgs) Handles AddHomeLogoBtn.Click, AddGuestLogoBtn.Click

        Dim logoFileDialog As New OpenFileDialog() With {
            .Title = "",
            .Filter = "JPEGs|*.jpg|GIFs|*.gif|Portable Graphics|*.png",
            .InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.MyPictures),
            .FilterIndex = 1,
            .RestoreDirectory = True
        }

        If logoFileDialog.ShowDialog() = DialogResult.OK Then
            If CType(sender, Button).Name = "AddHomeLogoBtn" Then
                With VideoPanel().HomeLogoPB
                    .Image = Image.FromFile(logoFileDialog.FileName)
                    .SizeMode = PictureBoxSizeMode.Zoom
                    .BorderStyle = BorderStyle.None
                End With
            ElseIf CType(sender, Button).Name = "AddGuestLogoBtn" Then
                With VideoPanel().GuestLogoPB
                    .Image = Image.FromFile(logoFileDialog.FileName)
                    .SizeMode = PictureBoxSizeMode.Zoom
                    .BorderStyle = BorderStyle.None
                End With
            Else
                Exit Sub
            End If
        End If
    End Sub

    Private Sub PlayerImportTSMI_Click(sender As Object, e As EventArgs) Handles HomePlayerImportTSMI.Click, GuestPlayerImportTSMI.Click
        Dim importFileDialog As New OpenFileDialog() With {
            .Title = "",
            .Filter = "comma separated (*.csv)|*.csv|semicolon separated (*.csv)|*.csv|Tab separated (*.txt)|*.txt|Pipe separated (*.txt)|*.txt",
            .InitialDirectory = Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments),
            .FilterIndex = 2,
            .RestoreDirectory = True
        }
        If importFileDialog.ShowDialog() = DialogResult.OK Then
            Using importSettingsDiag As New ImportPlayerSettings()
                importSettingsDiag.File2Import = importFileDialog.FileName
                Dim _dialogWindow As New Form() With {
                    .MaximizeBox = False,
                    .MinimizeBox = False,
                    .FormBorderStyle = FormBorderStyle.FixedDialog,
                    .AutoSize = True,
                    .StartPosition = FormStartPosition.CenterParent
                }
                _dialogWindow.Controls.Add(importSettingsDiag)
                _dialogWindow.Size = importSettingsDiag.Size
                Dim dlgResult As DialogResult = _dialogWindow.ShowDialog(Me)

                If dlgResult = DialogResult.OK Then
                    Me.ImportBW = New System.ComponentModel.BackgroundWorker() With {
                    .WorkerReportsProgress = True,
                    .WorkerSupportsCancellation = True
                }
                    AddHandler Me.ImportBW.DoWork, AddressOf ImportPlayerFile_Work
                    Me.ImportBW.RunWorkerAsync({
                                               importFileDialog.FileName,
                                               importSettingsDiag.GetFileDelimiter,
                                               importSettingsDiag.FileHasHeaders,
                                               importSettingsDiag.GetPlayerNumberColumnIndex,
                                               importSettingsDiag.GetPlayerNameColumnIndex,
                                               If(CType(sender, ToolStripMenuItem).Name.Contains("Home"), Team.Home, Team.Guest)
                                               })
                End If
            End Using
        End If

    End Sub

    Public Sub ImportPlayerFile_Work(sender As Object, e As System.ComponentModel.DoWorkEventArgs)
        Dim ImportFile As String = e.Argument(0)
        Dim ImportFilterDelimiter As String = e.Argument(1)
        Dim FileHasHeaders As Boolean = e.Argument(2)
        Dim PlayerNumberColIdx As Integer = e.Argument(3)
        Dim PlayerNameColIdx As Integer = e.Argument(4)
        Dim SelectedTeam As Team = e.Argument(5)
        Dim AddPlayerDelegate As New AddPlayerSafe(AddressOf AddPlayer)
        Using ImportFileReader As New FileIO.TextFieldParser(ImportFile)
            ImportFileReader.SetDelimiters(ImportFilterDelimiter)
            Dim currentRow As String()
            Dim headerRow As Boolean = True
            While Not ImportFileReader.EndOfData
                If ImportBW.CancellationPending Then
                    Exit While
                End If
                Try
                    currentRow = ImportFileReader.ReadFields()
                    If headerRow Then
                        headerRow = False
                    Else
                        Dim number As Integer = CInt(currentRow(PlayerNumberColIdx))
                        Dim name As String = currentRow(PlayerNameColIdx)
                        Dim pd As New Player(number, name, SelectedTeam)
                        Me.Invoke(AddPlayerDelegate, pd, SelectedTeam)
                    End If
                Catch ex As Exception
                    Debug.WriteLine(ex.Message)
                    ImportBW.CancelAsync()
                End Try
            End While
        End Using

        If ExportBW.CancellationPending Then
            Dim cancelDelegate As New BackgroundWorkerImportCompleteSafe(AddressOf ImportPlayerListComplete)
            Me.Invoke(cancelDelegate, True)
            Exit Sub
        End If
        Dim completeDelegate As New BackgroundWorkerImportCompleteSafe(AddressOf ImportPlayerListComplete)
        Me.Invoke(completeDelegate, False)
    End Sub

    Public Sub ImportPlayerListComplete(cancelled As Boolean)
        If cancelled Then
            MsgBox("Import Vorgang abgebrochen")
            Exit Sub
        End If

        With Me.BWPRogressBar
            .Visible = False
            .Value = 0
        End With
        HomePlayerListBS.Sort = "IsOfficial ASC, PlayerNumber ASC"
        Debug.WriteLine(HomePlayerListBS.SupportsAdvancedSorting)
         Debug.WriteLine(HomePlayerListBS.SupportsSorting)
        HomePlayerListBS.ResetBindings(False)
        GuestPlayerListBS.Sort = "IsOfficial ASC, PlayerNumber ASC"
        GuestPlayerListBS.ResetBindings(False)

        HomePlayerListDGV.Refresh()
        GuestPlayerListDGV.Refresh()
    End Sub

    Public Sub AddPlayer(row As Player, playerTeam As Team)
        Select Case playerTeam
            Case Team.Home
                With HomePlayerListBS
                    .Add(row)
                End With
            Case Team.Guest
                With GuestPlayerListBS
                    .Add(row)
                End With
        End Select
    End Sub

    Public Sub PlayerListDGV_CellContentClick(sender As Object, e As DataGridViewCellEventArgs) Handles HomePlayerListDGV.CellContentClick, GuestPlayerListDGV.CellContentClick
        Dim senderGrid = DirectCast(sender, DataGridView)

        If TypeOf senderGrid.Columns(e.ColumnIndex) Is DataGridViewButtonColumn AndAlso
       e.RowIndex >= 0 Then
            Dim playerData = DirectCast(senderGrid.Rows(e.RowIndex).DataBoundItem, Player)
            Select Case senderGrid.Columns(e.ColumnIndex).Name
                Case "HomePlayerWarningButtonColumn",
                 "GuestPlayerWarningButtonColumn"
                    If playerData.CountPenalties > 0 Then
                        senderGrid.Columns(e.ColumnIndex).Frozen = True
                        MsgBox(String.Format("Spieler hat bereits {0} Zeitstrafe(n) erhalten. Es kann keine Verwarngun mehr gegeben werden.", playerData.CountPenalties), MsgBoxStyle.Information)
                    Else
                        If Not playerData.PlayerHasWarning Then
                            playerData.SetPLayerWarning = CType(_videoForm, Anzeige).GetCurrentGameTime()
                            Dim gl As New GameLog(
                                String.Format(GlobalStrings.PlayerWarningMessage,
                                              playerData.PlayerName, playerData.PlayerNumber,
                                              If(playerData.PlayerTeam = Team.Home, GlobalStrings.Home, GlobalStrings.Guest)
                                              ),
                                CType(_videoForm, Anzeige).GetCurrentGameTime(),
                                playerData.PlayerTeam
                                )
                            With GamelogBS
                                .Add(gl)
                            End With
                        Else
                            MsgBox("Spieler hat bereits eine Verwarnung erhalten. Es kann keine weitere vergeben werden.", MsgBoxStyle.Information)
                            Exit Sub
                        End If
                    End If
                Case "HomePlayerPenaltyButtonColumn",
         "GuestPlayerPenaltyButtonColumn"
                    RaiseEvent OnStopGameTimer()
                    If playerData.CountPenalties < 3 Then
                        Dim newPenalty As New Penalty(CType(_videoForm, Anzeige).GetCurrentGameTime(), playerData.PlayerTeam, playerData.PlayerNumber, playerData.PlayerName, Scoreboard.Penalty.MIN2_PENALTY)
                        playerData.AddPenalty = newPenalty
                        HomePlayerListBS.ResetBindings(False)
                        Dim pe As New PenaltyEventArgs(newPenalty, False)
                        OnAddPenalty(sender, pe)
                    Else
                        MsgBox("Spieler wurde mit Rot vom Platz gestellt. Es ist keine weitere Strafzeit möglich")
                        Exit Sub
                    End If
            End Select
            CType(sender, DataGridView).Refresh()
        End If
    End Sub

    Private Sub PlayerListDGV_CellFormatting(sender As Object, e As DataGridViewCellFormattingEventArgs) Handles HomePlayerListDGV.CellFormatting, GuestPlayerListDGV.CellFormatting
        If e.ColumnIndex = DirectCast(sender, DataGridView).Columns(4).Index Then
            Dim data As Player = DirectCast(sender, DataGridView).Rows(e.RowIndex).DataBoundItem
            If data.IsOfficial AndAlso (Not data.PlayerHasWarning Or data.CountPenalties < 3) Then
                e.CellStyle.BackColor = Color.LightGray
            End If
            If (data.PlayerHasWarning And data.CountPenalties < 3) Then
                e.CellStyle.BackColor = Color.Yellow
            ElseIf (data.PlayerHasWarning And data.CountPenalties >= 3) Then
                e.CellStyle.BackColor = Color.Red
            ElseIf (Not data.PlayerHasWarning And data.CountPenalties >= 3) Then
                e.CellStyle.BackColor = Color.Red
            End If
            e.Value = String.Format("{0} ({1} Penalties)", e.Value, data.CountPenalties)
            e.FormattingApplied = True
        End If
        If e.ColumnIndex = DirectCast(sender, DataGridView).Columns(3).Index Then
            Dim data As Player = DirectCast(sender, DataGridView).Rows(e.RowIndex).DataBoundItem
            If data.IsOfficial Then
                e.Value = data.OfficlaNumber
            Else
                e.Value = data.PlayerNumber.ToString
            End If
            e.FormattingApplied = True
        End If
        If e.ColumnIndex = DirectCast(sender, DataGridView).Columns(8).Index Then
            Dim data As Player = DirectCast(sender, DataGridView).Rows(e.RowIndex).DataBoundItem
            If (data.CountPenalties >= 3) Then
                e.CellStyle.BackColor = Color.Gray
                e.FormattingApplied = True
            End If
        End If
        If e.ColumnIndex = DirectCast(sender, DataGridView).Columns(7).Index Then
            Dim data As Player = DirectCast(sender, DataGridView).Rows(e.RowIndex).DataBoundItem
            If (data.PlayerHasWarning) Then
                e.CellStyle.BackColor = Color.Gray
                e.FormattingApplied = True
            End If
        End If

    End Sub

    Private Sub PlayerListRemoveTSMI_Click(sender As Object, e As EventArgs) Handles HomePlayerListRemoveTSMI.Click, GuestPlayerListRemoveTSMI.Click
        If CType(sender, ToolStripMenuItem).Name.Contains("Home") Then
            HomePlayerListBS.Clear()
            HomePlayerListBS.ResetBindings(False)
        ElseIf CType(sender, ToolStripMenuItem).Name.Contains("Guest") Then
            GuestPlayerListBS.Clear()
            GuestPlayerListBS.ResetBindings(False)
        End If
    End Sub
End Class
