﻿Imports Scoreboard.My.Resources

Public Class GameControl
    Dim _dialogCtrl As Control = Nothing
    Dim _dialogWindow As Form = Nothing
    Dim _timerStopped As Boolean
    Dim _currentTime As Integer = 0
    Dim _parentControlForm As Form = Nothing
    Dim _currentTimeChanged As Boolean
    Public Event OnChangeCurrentGameTimeByUser(e As GameTimeEventArgs)

    Public Enum Team
        Home
        Guest
    End Enum

    Public Property TimerStopped As Boolean
        Get
            Return _timerStopped
        End Get
        Set(value As Boolean)
            _timerStopped = value
            RaiseEvent OnTimerEnabled()
        End Set
    End Property

    Public WriteOnly Property CurrentTime() As Integer
        Set(value As Integer)
            _currentTime = value
        End Set
    End Property

    Private ReadOnly Property GetCurrentTime() As Integer
        Get
            Return _currentTime
        End Get
    End Property

    Private Property ParentControlForm() As Form
        Get
            Return _parentControlForm
        End Get
        Set(value As Form)
            _parentControlForm = value
        End Set
    End Property

    Private Property CurrentTimeChanged() As Boolean
        Get
            Return _currentTimeChanged
        End Get
        Set(value As Boolean)
            _currentTimeChanged = value
        End Set
    End Property

    Public Event OnRaiseAddScore(selectedTeam As Integer)
    Public Event OnRaiseSubScore(selectedTeam As Integer)
    Public Event OnRaiseAddPenalty()
    Public Event OnRaiseTimeout(selectedTeam As Integer)
    Public Event OnRaiseStopClock()
    Public Event OnRaiseToggleClock()
    Public Event OnRaiseTeamPenalty As EventHandler
    Public Event OnTimerEnabled()

    Private Sub GameControl_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        ParentControlForm = FindForm()

        Me.HomeAddPenaltyBtn.TabStop = False
        Me.GuestAddPenaltyBtn.TabStop = False

        Select Case TypeOf ParentControlForm Is Bedienung
            Case True
                AddHandler DirectCast(ParentControlForm, Bedienung).VideoPanel().OnGameTimerTick, AddressOf RefreshGameTime
                CurrentTime = DirectCast(ParentControlForm, Bedienung).VideoPanel().GetCurrentGameTime
                RefreshGameTime()
        End Select
    End Sub

    Private Sub HomeAddScoreBtn_Click(sender As Object, e As EventArgs) Handles HomeAddScoreBtn.Click
        RaiseEvent OnRaiseAddScore(Team.Home)

    End Sub

    Private Sub HomeSubScoreBtn_Click(sender As Object, e As EventArgs) Handles HomeSubScoreBtn.Click
        RaiseEvent OnRaiseSubScore(Team.Home)
    End Sub

    Private Sub HomeAddPenaltyBtn_Click(sender As Object, e As EventArgs) Handles HomeAddPenaltyBtn.Click
        RaiseEvent OnRaiseStopClock()
        _dialogWindow = New Form()
        With _dialogWindow
            .MaximizeBox = False
            .MinimizeBox = False
            .FormBorderStyle = FormBorderStyle.FixedDialog
            .AutoSize = True
            .Text = "Add Penalty for Player - Home Team"
            .StartPosition = FormStartPosition.CenterParent
        End With
        _dialogCtrl = New SelectTeamDialog(Team.Home)
        AddHandler CType(_dialogCtrl, SelectTeamDialog).onSelectTeamPenalty, AddressOf OnSelectPenalty
        _dialogWindow.Controls.Add(_dialogCtrl)
        _dialogWindow.Size = _dialogCtrl.Size
        _dialogWindow.ShowDialog(Me)
    End Sub

    Private Sub GuestAddScoreBtn_Click(sender As Object, e As EventArgs) Handles GuestAddScoreBtn.Click
        RaiseEvent OnRaiseAddScore(Team.Guest)
    End Sub

    Private Sub GuestSubScoreBtn_Click(sender As Object, e As EventArgs) Handles GuestSubScoreBtn.Click
        RaiseEvent OnRaiseSubScore(Team.Guest)
    End Sub

    Private Sub StartStopBtn_Click(sender As Object, e As EventArgs) Handles StartStopBtn.Click
        If TypeOf ParentControlForm Is Bedienung And CType(ParentControlForm, Bedienung).VideoPanel.TimeOutTimer.Enabled Then
            Return
        End If
        RaiseEvent OnRaiseToggleClock()
    End Sub

    Private Sub GuestAddPenaltyBtn_Click(sender As Object, e As EventArgs) Handles GuestAddPenaltyBtn.Click
        RaiseEvent OnRaiseStopClock()
        _dialogWindow = New Form()
        With _dialogWindow
            .MaximizeBox = False
            .MinimizeBox = False
            .FormBorderStyle = FormBorderStyle.FixedDialog
            .AutoSize = True
            .Text = "Add Penalty for Player - Guest Team"
            .StartPosition = FormStartPosition.CenterParent
        End With
        _dialogCtrl = New SelectTeamDialog(Team.Guest)
        AddHandler CType(_dialogCtrl, SelectTeamDialog).onSelectTeamPenalty, AddressOf OnSelectPenalty
        _dialogWindow.Controls.Add(_dialogCtrl)
        _dialogWindow.Size = _dialogCtrl.Size
        _dialogWindow.ShowDialog(Me)
    End Sub

    Private Sub OnSelectPenalty(sender As Object, e As PenaltyEventArgs)
        If _dialogWindow IsNot Nothing Then
            _dialogWindow.Close()
            _dialogWindow.Dispose()
            _dialogWindow = Nothing
            RaiseEvent OnRaiseTeamPenalty(sender, e)
        End If
    End Sub

    Private Sub TimerEnabled() Handles Me.OnTimerEnabled
        If (TimerStopped) Then
            With GameTimeMTB
                .Enabled = True
            End With
            With Me.StartStopBtn
                .Text = "Start"
                .ForeColor = Color.White
                .BackColor = Color.OrangeRed
            End With
            CheckCurrentTimeChanged()
        Else
            With GameTimeMTB
                .Enabled = False
            End With
            With Me.StartStopBtn
                .Text = "Stop"
                .ForeColor = Color.Black
                .BackColor = Color.ForestGreen
            End With
        End If
    End Sub



    Public Sub RefreshGameTime()
        CurrentTime = DirectCast(ParentControlForm, Bedienung).VideoPanel().GetCurrentGameTime
        Dim Minutes As Integer = 0
        Dim Seconds As Integer = GetCurrentTime / 1000
        If (Seconds > 59) Then
            Dim _convTime2Minutes = Seconds / 60
            Minutes = Convert.ToInt16(_convTime2Minutes.ToString.Split(",")(0))
            Seconds -= (Minutes * 60)
        End If

        With GameTimeMTB
            .Text = Minutes.ToString.PadLeft(2, "0") & Seconds.ToString.PadLeft(2, "0")
        End With
    End Sub

    Private Function CheckCurrentTimeChanged()
        Dim newTimeStr As String = GameTimeMTB.Text
        If (newTimeStr.Length < 5) Then
            Return False
        End If
        Dim _minutes As Integer = Convert.ToInt16(newTimeStr.Substring(0, 2))
        Dim _seconds As Integer = Convert.ToInt16(newTimeStr.Substring(3, 2))
        Dim newTime As Integer = ((_minutes * 60) + _seconds) * 1000

        CurrentTimeChanged = newTime <> GetCurrentTime()

        If CurrentTimeChanged Then
            Return True
        Else
            Return False
        End If
    End Function


    Private Sub BuzzerBtn_Click(sender As Object, e As EventArgs) Handles BuzzerBtn.Click
        My.Computer.Audio.Play(My.Resources.BUZZER, AudioPlayMode.Background)
    End Sub

    Private Sub TimeoutStarted(selTeam As Integer) Handles Me.OnRaiseTimeout
        DeactivateGamePanelControls()
    End Sub

    Private Sub DeactivateGamePanelControls()
        StartStopBtn.Enabled = False
        AddTTOGuestBtn.Enabled = False
        AddTTOHomeBtn.Enabled = False
    End Sub

    Private Sub GameTimeMTB_KeyUp(sender As Object, e As KeyEventArgs) Handles GameTimeMTB.KeyUp
        If (CType(sender, MaskedTextBox).Text.Length < 5) Then
            Return
        End If
        If CheckCurrentTimeChanged() Then
            CallChangeTimeDialog()
        End If
    End Sub

    Private Sub CallChangeTimeDialog()
        Dim diagRes As DialogResult = MessageBox.Show(GlobalStrings.ChangeGameTimeMessage, GlobalStrings.ChangeGameTime, MessageBoxButtons.YesNo)

        If (diagRes = DialogResult.Yes) Then
            Dim newTimeStr As String = GameTimeMTB.Text
            Dim _minutes As Integer = Convert.ToInt16(newTimeStr.Substring(0, 2))
            Dim _seconds As Integer = Convert.ToInt16(newTimeStr.Substring(3, 2))
            Dim newTime As Integer = ((_minutes * 60) + _seconds) * 1000
            Dim gtEventArgs As New GameTimeEventArgs(newTime, GetCurrentTime)
            RaiseEvent OnChangeCurrentGameTimeByUser(gtEventArgs)
        End If
    End Sub

    Private Sub AddTTOHomeBtn_Click(sender As Object, e As EventArgs) Handles AddTTOHomeBtn.Click
        If CType(ParentControlForm, Bedienung).VideoPanel.TimeOutTimer.Enabled = True Then
            Return
        End If
        RaiseEvent OnRaiseStopClock()
        My.Computer.Audio.Play(My.Resources.BUZZER, AudioPlayMode.Background)
        Dim dlg As DialogResult = MessageBox.Show(GlobalStrings.TimeoutHomeMessage, GlobalStrings.TimeOutHome, MessageBoxButtons.YesNo)

        If (dlg = DialogResult.Yes) Then
            RaiseEvent OnRaiseTimeout(Team.Home)
        End If

    End Sub

    Private Sub AddTTOGuestBtn_Click(sender As Object, e As EventArgs) Handles AddTTOGuestBtn.Click
        If CType(ParentControlForm, Bedienung).VideoPanel.TimeOutTimer.Enabled = True Then
            Return
        End If
        RaiseEvent OnRaiseStopClock()
        My.Computer.Audio.Play(My.Resources.BUZZER, AudioPlayMode.Background)
        Dim dlg As DialogResult = MessageBox.Show(GlobalStrings.TimeoutGuestMessage, GlobalStrings.TimeOutGuest, MessageBoxButtons.YesNo)

        If (dlg = DialogResult.Yes) Then
            RaiseEvent OnRaiseTimeout(Team.Guest)
        End If
    End Sub

End Class
