﻿

Imports Scoreboard.My.Resources

<Serializable>
Public Class GameLog
    ReadOnly _ID As String
    ReadOnly _logTime As DateTime
    ReadOnly _Message As String
    ReadOnly _GameTime As Integer
    ReadOnly _Team As Integer
    ReadOnly _HomeScore As Integer
    ReadOnly _GuestScore As Integer
    ReadOnly _Penalty As Penalty
    ReadOnly _Period As Integer
    ReadOnly _GameWarning As Integer

    Public ReadOnly Property GetID() As String
        Get
            Return _ID
        End Get
    End Property

    Public ReadOnly Property GetLogTime() As DateTime
        Get
            Return _logTime
        End Get
    End Property

    Public ReadOnly Property GetMessage() As String
        Get
            Return _Message
        End Get
    End Property

    Public ReadOnly Property GetGameTime() As Integer
        Get
            Return _GameTime
        End Get
    End Property

    Public ReadOnly Property GetTeam() As Integer
        Get
            Return _Team
        End Get
    End Property

    Public ReadOnly Property GetTeamName() As String
        Get
            Return If(_Team = Team.Home, GlobalStrings.Home, GlobalStrings.Guest)
        End Get
    End Property

    Public ReadOnly Property GetHomeScore() As Integer
        Get
            Return _HomeScore
        End Get
    End Property

    Public ReadOnly Property GetGuestScore() As Integer
        Get
            Return _GuestScore
        End Get
    End Property

    Public ReadOnly Property GetGameScore() As String
        Get
            Return String.Format("{0}:{1}", _HomeScore.ToString.PadLeft(2, "0"), _GuestScore.ToString.PadLeft(2, "0"))
        End Get
    End Property

    Public ReadOnly Property GetPenalty() As Penalty
        Get
            Return _Penalty
        End Get
    End Property

    Public ReadOnly Property GetPeriod() As Integer
        Get
            Return _Period
        End Get
    End Property

    Public ReadOnly Property GetWarning() As Integer
        Get
            Return _GameWarning
        End Get
    End Property

    Public ReadOnly Property GetWarningTime() As String
        Get
            Return ConvertTimeToString(_GameWarning)
        End Get
    End Property

    Public ReadOnly Property GetLog() As String
        Get
            Return String.Format("{0} {1}|{2}|{3}|{4}|{5}|{6}",
                                 _logTime.ToShortDateString,
                                 _logTime.ToShortTimeString,
                                 ConvertTimeToString(_GameTime),
                                 GetGameScore,
                                 _Message,
                                 If((_Team = Team.Home), GlobalStrings.Home, GlobalStrings.Guest),
                                 If((_Penalty IsNot Nothing), String.Format("{0} ({1})", ConvertTimeToString(_Penalty.ReentryTime), _Penalty.PlayerNumber), "")
                                 )
        End Get
    End Property

    Public Sub New()
        _ID = Guid.NewGuid().ToString
        _logTime = DateTime.Now
    End Sub

    ''' <summary>
    ''' Add new Gamelog with message and game time. optional team can be set
    ''' </summary>
    ''' <param name="msg"></param>
    ''' <param name="gameTime"></param>
    ''' <param name="selTeam"></param>
    Public Sub New(msg As String, gameTime As Integer, Optional selTeam As Team = Nothing)
        _ID = Guid.NewGuid().ToString
        _logTime = DateTime.Now
        _Message = msg
        _GameTime = gameTime

        If selTeam Then
            _Team = selTeam
        End If
    End Sub

    ''' <summary>
    ''' Add new Gamelog with current game time and who has scored inluding points
    ''' </summary>
    ''' <param name="gameTime"></param>
    ''' <param name="scoreTeam"></param>
    ''' <param name="homePoints"></param>
    ''' <param name="guestPoints"></param>
    Public Sub New(gameTime As Integer, scoreTeam As Integer, homePoints As Integer, guestPoints As Integer)
        _ID = Guid.NewGuid().ToString
        _logTime = DateTime.Now
        _GameTime = gameTime
        _HomeScore = homePoints
        _GuestScore = guestPoints
        _Team = scoreTeam
        _Message = String.Format(GlobalStrings.LogScoreMessage, If((_Team = Team.Home), GlobalStrings.Home, GlobalStrings.Guest), _HomeScore, _GuestScore)
    End Sub

    ''' <summary>
    ''' Add new Gamelog with penalty and points
    ''' </summary>
    ''' <param name="gameTime"></param>
    ''' <param name="penaltyTime"></param>
    ''' <param name="homePoints"></param>
    ''' <param name="guestPoints"></param>
    Public Sub New(gameTime As Integer, penaltyTime As Penalty, homePoints As Integer, guestPoints As Integer)
        _ID = Guid.NewGuid().ToString
        _logTime = DateTime.Now
        _GameTime = gameTime
        _Penalty = penaltyTime
        _Team = penaltyTime.PlayerTeam
        _HomeScore = homePoints
        _GuestScore = guestPoints
        _Message = String.Format(GlobalStrings.LogPenaltyMessage, If((penaltyTime.PlayerName.Length > 0), penaltyTime.PlayerName, penaltyTime.PlayerNumber), If((penaltyTime.PlayerTeam = Team.Home), GlobalStrings.Home, GlobalStrings.Guest))
    End Sub

    ''' <summary>
    ''' Add new Gamelog with message and points
    ''' </summary>
    ''' <param name="msg"></param>
    ''' <param name="gameTime"></param>
    ''' <param name="homePoints"></param>
    ''' <param name="guestPoints"></param>
    Public Sub New(msg As String, gameTime As Integer, homePoints As Integer, guestPoints As Integer, period As Integer)
        _ID = Guid.NewGuid().ToString
        _logTime = DateTime.Now
        _Message = msg
        _GameTime = gameTime
        _HomeScore = homePoints
        _GuestScore = guestPoints
        _Period = period
    End Sub

    Friend Shared Function ConvertTimeToString(ByVal currentTime As Integer) As String
        Dim Minutes As Integer = 0
        Dim Seconds As Integer = currentTime / 1000
        If (Seconds > 59) Then
            Dim _convTime2Minutes = Seconds / 60
            Minutes = Convert.ToInt16(_convTime2Minutes.ToString.Split(",")(0))
            Seconds -= (Minutes * 60)
        End If
        Return String.Format("{0}:{1}", Minutes.ToString.PadLeft(2, "0"), Seconds.ToString.PadLeft(2, "0"))
    End Function
End Class