﻿Imports MigraDoc.DocumentObjectModel

Public Class PdfProtocoll
    Private ReadOnly _document As Document
    Private _Section As Section

    Public ReadOnly Property PdfDocument() As Document
        Get
            Return _document
        End Get
    End Property

    Public Sub New()
        _document = New Document()
    End Sub
    Public Sub New(Title As String)
        _document = New Document()
        _document.Info.Title = Title
        InitDocument()
    End Sub
    Public Sub New(Title As String, Subject As String)
        _document = New Document()
        _document.Info.Title = Title
        _document.Info.Subject = Subject
        InitDocument()
    End Sub
    Public Sub New(Title As String, Subject As String, Author As String)
        _document = New Document()
        _document.Info.Title = Title
        _document.Info.Subject = Subject
        _document.Info.Author = Author
        InitDocument()
    End Sub

    Private Sub InitDocument()
        With _document
            .DefaultPageSetup.PageFormat = PageFormat.A4
            .DefaultPageSetup.Orientation = Orientation.Landscape
            .DefaultPageSetup.TopMargin = Unit.FromCentimeter(1)
            .DefaultPageSetup.RightMargin = Unit.FromCentimeter(1)
            .DefaultPageSetup.BottomMargin = Unit.FromCentimeter(1)
            .DefaultPageSetup.LeftMargin = Unit.FromCentimeter(1)
        End With

        Me._Section = _document.AddSection()
        With Me._Section
            .PageSetup = _document.DefaultPageSetup.Clone()
            .PageSetup.PageFormat = PageFormat.A4
            .PageSetup.Orientation = Orientation.Portrait
        End With
        DefineStyles()
    End Sub

    Private Sub DefineStyles()
        Dim style As Style = _document.Styles(StyleNames.Normal)
        style.Font.Name = "Verdana"

        style = _document.Styles(StyleNames.Header)
        style.ParagraphFormat.AddTabStop("2cm", TabAlignment.Right)

        style = _document.Styles(StyleNames.Footer)
        style.ParagraphFormat.AddTabStop("2cm", TabAlignment.Center)

        style = _document.Styles.AddStyle("Table", StyleNames.Normal)
        style.Font.Name = "Verdana"
        'style.Font.Name = "Times New Roman"
        style.Font.Size = 8
    End Sub

    Public Sub AddGameData(GameTitle As String, HomeTeamName As String, GuestTeamName As String, IsCupMatch As Boolean)
        Dim _paragraph = Me._Section.AddParagraph()
        With _paragraph
            .AddFormattedText("Game Title:", TextFormat.Bold)
            .AddTab()
            .AddText(GameTitle)
            .AddLineBreak()
            .AddLineBreak()
        End With
        _paragraph = Me._Section.AddParagraph()
        With _paragraph
            .AddFormattedText("Game Info:", TextFormat.Bold)
            .AddTab()
            .AddText(HomeTeamName)
            .AddTab()
            .AddText("vs.")
            .AddTab()
            .AddText(GuestTeamName)
            .AddLineBreak()
            .AddLineBreak()
        End With

        If IsCupMatch Then
            With _paragraph
                .AddFormattedText("Cup Match", TextFormat.Bold)
                .AddLineBreak()
                .AddLineBreak()
            End With
        End If
    End Sub

    Public Sub AddEntries(LogBS As BindingSource)
        If (LogBS Is Nothing) Then
            Exit Sub
        End If

        Dim _paragraph = Me._Section.AddParagraph()
        With _paragraph
            .AddFormattedText("Game Protocoll", TextFormat.Bold)
            .AddLineBreak()
            .AddLineBreak()
        End With
        Dim _table As Tables.Table = Me._Section.AddTable()
        With _table
            .Borders.Width = 0.25
            .Rows.LeftIndent = -10
            .Style = "Table"
        End With
        Dim _column As Tables.Column = _table.AddColumn(Unit.FromCentimeter(1.8)) ' Spielzeit
        _column.Format.Alignment = ParagraphAlignment.Center
        _column = _table.AddColumn(Unit.FromCentimeter(1.8))    ' Spielstand
        _column.Format.Alignment = ParagraphAlignment.Center
        _column = _table.AddColumn(Unit.FromCentimeter(7))      ' Nachricht
        _column.Format.Alignment = ParagraphAlignment.Left
        _column = _table.AddColumn(Unit.FromCentimeter(1))      ' Team
        _column.Format.Alignment = ParagraphAlignment.Center
        _column = _table.AddColumn(Unit.FromCentimeter(2.6))    ' Strafzeit(en)
        _column.Format.Alignment = ParagraphAlignment.Center
        _column = _table.AddColumn(Unit.FromCentimeter(3))      ' Logzeit
        _column.Format.Alignment = ParagraphAlignment.Center
        Dim _row As Tables.Row = _table.AddRow()
        _row.HeadingFormat = True
        _row.Shading.Color = Colors.LightGray
        Dim _cell As Tables.Cell = _row.Cells(0)    ' Spielzeit
        _cell.AddParagraph(My.Resources.GlobalStrings.GameTime)
        _cell = _row.Cells(1)                       ' Spielstand
        _cell.AddParagraph(My.Resources.GlobalStrings.GameScore)
        _cell = _row.Cells(2)                       ' Nachricht
        _cell.AddParagraph(My.Resources.GlobalStrings.Message)
        _cell = _row.Cells(3)                       ' Mannschaft (Heim || Gast)
        _cell.AddParagraph("T")
        _cell = _row.Cells(4)                       ' Strafzeit(en)
        _cell.AddParagraph(My.Resources.GlobalStrings.Penalties)
        _cell = _row.Cells(5)                       ' Logzeit
        _cell.AddParagraph(My.Resources.GlobalStrings.LogTime)

        For Each Item As GameLog In LogBS.List
            _row = _table.AddRow()
            _cell = _row.Cells(0)                       ' Spielzeit
            _cell.AddParagraph(GameLog.ConvertTimeToString(Item.GetGameTime))
            _cell = _row.Cells(1)                       ' Spielstand
            _cell.AddParagraph(Item.GetGameScore)
            _cell = _row.Cells(2)                       ' Nachricht
            _cell.AddParagraph(Item.GetMessage)
            _cell = _row.Cells(3)                       ' Mannschaft
            _cell.AddParagraph(If(Item.GetTeam = Team.Home, "H", "G"))
            _cell = _row.Cells(4)                       ' Strafzeit(en)
            _cell.AddParagraph(If((Item.GetPenalty() IsNot Nothing), String.Format("{0} ({1})", GameLog.ConvertTimeToString(Item.GetPenalty().ReentryTime), Item.GetPenalty().PlayerNumber), ""))
            _cell = _row.Cells(5)                       ' Logzeit
            _cell.AddParagraph(Item.GetLogTime.ToShortDateString + " " + Item.GetLogTime.ToShortTimeString)
        Next
        _table.SetEdge(0, 0, 6, 1, Tables.Edge.Box, BorderStyle.Single, 0.5, Colors.Black)
    End Sub

    Public Sub AddPLayerList(PlayerBS As BindingSource, PlayerTeam As Team, Optional createNewSection As Boolean = False)
        If (PlayerBS Is Nothing) Then
            Exit Sub
        End If
        If createNewSection Then
            Me._Section = _document.AddSection
            With Me._Section
                .PageSetup = _document.DefaultPageSetup.Clone()
                .PageSetup.PageFormat = PageFormat.A4
                .PageSetup.Orientation = Orientation.Portrait
            End With
            DefineStyles()
        Else
            Me._Section = _document.LastSection()
        End If

        Dim _paragraph = Me._Section.AddParagraph()
        With _paragraph
            .AddFormattedText(String.Format("Player List for {0} Team", (If(PlayerTeam = Team.Home, "Home", "Guest"))), TextFormat.Bold)
            .AddLineBreak()
            .AddLineBreak()
        End With
        Dim _table As Tables.Table = Me._Section.AddTable()
        With _table
            .Borders.Width = 0.25
            .Rows.LeftIndent = -10
            .Style = "Table"
        End With
        Dim _column As Tables.Column = _table.AddColumn(Unit.FromCentimeter(2)) ' 1 Player Name
        _column.Format.Alignment = ParagraphAlignment.Center
        _column = _table.AddColumn(Unit.FromCentimeter(1.0))    ' 2 Warning
        _column.Format.Alignment = ParagraphAlignment.Center
        _column = _table.AddColumn(Unit.FromCentimeter(1.0))    ' 3 first penalty
        _column.Format.Alignment = ParagraphAlignment.Center
        _column = _table.AddColumn(Unit.FromCentimeter(1.0))    ' 4 second penalty
        _column.Format.Alignment = ParagraphAlignment.Center
        _column = _table.AddColumn(Unit.FromCentimeter(1.0))    ' 5 third penalty
        _column.Format.Alignment = ParagraphAlignment.Center
        _column = _table.AddColumn(Unit.FromCentimeter(1.0))    ' 6 disqualification
        _column.Format.Alignment = ParagraphAlignment.Center
        _column = _table.AddColumn(Unit.FromCentimeter(1.0))    ' 7 exclusion
        _column.Format.Alignment = ParagraphAlignment.Center
        Dim _row As Tables.Row = _table.AddRow()
        _row.HeadingFormat = True
        _row.Shading.Color = Colors.LightGray
        Dim _cell As Tables.Cell = _row.Cells(0)    ' Player Name
        _cell.AddParagraph(My.Resources.GlobalStrings.PlayerName)
        _cell = _row.Cells(1)                       ' Warning
        _cell.AddParagraph(My.Resources.GlobalStrings.PlayerWarning)
        _cell = _row.Cells(2)                       ' first penalty
        _cell.AddParagraph(My.Resources.GlobalStrings.Penalty)
        _cell = _row.Cells(3)                       ' second penalty
        _cell.AddParagraph(My.Resources.GlobalStrings.Penalty)
        _cell = _row.Cells(4)                       ' third penalty
        _cell.AddParagraph(My.Resources.GlobalStrings.Penalty)
        _cell = _row.Cells(5)                       ' disqualification
        _cell.AddParagraph(My.Resources.GlobalStrings.Disqualification)
        _cell = _row.Cells(6)                       ' exclusion
        _cell.AddParagraph(My.Resources.GlobalStrings.Exclusion)

        For Each Item As Player In PlayerBS.List
            _row = _table.AddRow()
            _cell = _row.Cells(0)                       ' Player Name
            _cell.AddParagraph(Item.PlayerName)
            _cell = _row.Cells(1)                       ' Warning
            _cell.AddParagraph(If(Item.PlayerWarning, GameLog.ConvertTimeToString(Item.PlayerWarning), ""))
            Select Case Item.CountPenalties
                Case 0
                    _cell = _row.Cells(2)                       ' no penalties
                    _cell.AddParagraph("")
                    _cell = _row.Cells(3)                       ' no penalties
                    _cell.AddParagraph("")
                    _cell = _row.Cells(4)                       ' no penalties
                    _cell.AddParagraph("")
                Case 1
                    _cell = _row.Cells(2)                       ' first penalty
                    _cell.AddParagraph(GameLog.ConvertTimeToString(Item.GetPenalties(0).GameTime))
                    cell = _row.Cells(3)                       ' no penalties
                    _cell.AddParagraph("")
                    _cell = _row.Cells(4)                       ' no penalties
                    _cell.AddParagraph("")
                Case 2
                    _cell = _row.Cells(2)                       ' first penalty
                    _cell.AddParagraph(GameLog.ConvertTimeToString(Item.GetPenalties(0).GameTime))
                    _cell = _row.Cells(3)                       ' first penalty
                    _cell.AddParagraph(GameLog.ConvertTimeToString(Item.GetPenalties(1).GameTime))
                    _cell = _row.Cells(4)                       ' no penalties
                    _cell.AddParagraph("")
                Case 3
                    _cell = _row.Cells(2)                       ' first penalty
                    _cell.AddParagraph(GameLog.ConvertTimeToString(Item.GetPenalties(0).GameTime))
                    _cell = _row.Cells(3)                       ' first penalty
                    _cell.AddParagraph(GameLog.ConvertTimeToString(Item.GetPenalties(1).GameTime))
                    _cell = _row.Cells(4)                       ' first penalty
                    _cell.AddParagraph(GameLog.ConvertTimeToString(Item.GetPenalties(2).GameTime))
                    _cell = _row.Cells(5)                       ' disqualification
                    _cell.AddParagraph(GameLog.ConvertTimeToString(Item.GetPenalties(2).GameTime))
            End Select
        Next
    End Sub
End Class
