﻿Imports System.IO
Imports System.Security
Imports Newtonsoft.Json

Public Class Settings
    Private _activeTab As TabPage = Nothing
    Private _confFiles As List(Of String)
    Private _loadConfig As Configuration = Nothing
    Private _editConfiguration As Boolean = False

    Public ReadOnly Property ConfigFiles() As List(Of String)
        Get
            Return _confFiles
        End Get
    End Property

    Public WriteOnly Property LoadConfiguration() As Configuration
        Set(value As Configuration)
            _loadConfig = value
        End Set
    End Property

    Public Property EditConfiguration() As Boolean
        Get
            Return _editConfiguration
        End Get
        Set(value As Boolean)
            _editConfiguration = value
        End Set
    End Property

    Public Sub OnStart(sender As Object, e As EventArgs) Handles Me.Load
        With UserConfigurationsLB
            .DisplayMember = "Name"
            .ValueMember = ""
        End With
        If SettingsTabControl.SelectedTab IsNot Nothing Then
            _activeTab = SettingsTabControl.SelectedTab
            InitVisibleButtons()
        End If

        _confFiles = New List(Of String)
        readConfigFiles()
        If ConfigFiles.Count > 0 Then
            InitJsonSettings()
        End If
        Me.runSettingsScreenAlwaysCB.Checked = My.Settings.runSettingsScreenonStart
        GameTimeTB.Text = String.Format(New PenaltyTimeFormatProvider(), "{0:U}", My.Settings.gameTime)
        HalfTimeTB.Text = String.Format(New PenaltyTimeFormatProvider(), "{0:U}", My.Settings.halfTime)
        TimeOutTB.Text = String.Format(New PenaltyTimeFormatProvider(), "{0:U}", My.Settings.timeoutTime)
        extraTimeTB.Text = String.Format(New PenaltyTimeFormatProvider(), "{0:U}", My.Settings.extraTime)
        extraTimeHTTB.Text = String.Format(New PenaltyTimeFormatProvider(), "{0:U}", My.Settings.extraHalftime)
        GamePeriodsUD.Value = My.Settings.gamePeriods
        extraTimeRoundsUD.Value = My.Settings.extraTimeRounds
        CheckBox1.Checked = My.Settings.runSettingsScreenonStart
    End Sub

    Private Sub CancelBtn_Click(sender As Object, e As EventArgs) Handles CancelBtn.Click
        Me.DialogResult = DialogResult.Cancel
        Me.Close()
    End Sub

    Private Sub OkBtn_Click(sender As Object, e As EventArgs) Handles OkBtn.Click

        Select Case _activeTab.Name
            Case "General"
                My.Settings.gameTime = convertTime(GameTimeTB.Text)
                My.Settings.halfTime = convertTime(HalfTimeTB.Text)
                My.Settings.timeoutTime = convertTime(TimeOutTB.Text)
                My.Settings.gamePeriods = GamePeriodsUD.Value
                My.Settings.extraTime = convertTime(extraTimeTB.Text)
                My.Settings.extraHalftime = convertTime(extraTimeHTTB.Text)
                My.Settings.extraTimeRounds = extraTimeRoundsUD.Value
                My.Settings.runSettingsScreenonStart = CheckBox1.Checked
                My.Settings.Save()
            Case "Custom"
                If Not UserConfigurationsLB.SelectedIndex = -1 Then
                    Dim conf As Configuration = UserConfigurationsLB.SelectedItem
                    My.Settings.gameTime = convertTime(conf.Settings.gameTime)
                    My.Settings.halfTime = convertTime(conf.Settings.halfTime)
                    My.Settings.timeoutTime = convertTime(conf.Settings.timeoutTime)
                    My.Settings.gamePeriods = conf.Settings.gamePeriods
                    My.Settings.extraTime = convertTime(conf.Settings.extraTime)
                    My.Settings.extraHalftime = convertTime(conf.Settings.extraHalftime)
                    My.Settings.extraTimeRounds = conf.Settings.extraTimeRounds
                    My.Settings.runSettingsScreenonStart = CheckBox1.Checked
                    My.Settings.Save()
                Else
                    MsgBox("Es wurde keine Bentuzerkonfiguration ausgewählt." & vbCrLf & "Es wird mit den Allgemeinen Einstellungen fortgefahren.", MsgBoxStyle.Information, "Bentuzerkonfiguration übernehmen")
                End If
        End Select
        Me.DialogResult = DialogResult.OK
        Me.Close()
    End Sub

    Private Function ConvertTime(content As String) As Integer
        Dim newTimeStr As String = content
        Dim _minutes As Integer = Convert.ToInt16(newTimeStr.Substring(0, 2))
        Dim _seconds As Integer = Convert.ToInt16(newTimeStr.Substring(3, 2))
        Dim newTime As Integer = ((_minutes * 60) + _seconds) * 1000
        Return newTime
    End Function

    Private Sub ReadConfigFiles()
        _confFiles.Clear()
        Dim _dataPath As String = My.Application.Info.DirectoryPath & "\\data"
        If Not Directory.Exists(_dataPath) Then
            Directory.CreateDirectory(_dataPath)
        End If
        Dim di As New DirectoryInfo(_dataPath)
        Dim fi As FileInfo
        Dim i As Integer = 0
        For Each fi In di.GetFiles("*.sbs")
            _confFiles.Add(fi.FullName)
            i += 1
        Next
    End Sub

    Private Sub InitJsonSettings()

        Dim jS As String
        For Each file As String In ConfigFiles
            Dim _path As String = Path.Combine(My.Application.Info.DirectoryPath, "data", file)
            Console.WriteLine(file)
            Try
                jS = IO.File.ReadAllText(_path)
                Dim conf As Configuration = JsonConvert.DeserializeObject(Of Configuration)(jS)
                With UserConfigurationsLB
                    .Items.Clear()
                    .Items.Add(conf)
                End With
            Catch ex As Exception
                MsgBox(ex.Message)
            End Try
        Next
    End Sub

    Private Sub UserConfigurationsLB_SelectedIndexChanged(sender As Object, e As EventArgs) Handles UserConfigurationsLB.SelectedIndexChanged
        If Not _activeTab.Name = "Custom" Then
            Exit Sub
        End If

        If CType(sender, ListBox).SelectedIndex = -1 Then
            EditConfigurationBtn.Enabled = False
            Return
        Else
            EditConfigurationBtn.Enabled = True
        End If
    End Sub

    Private Sub SettingsTabControl_TabIndexChanged(sender As Object, e As EventArgs) Handles SettingsTabControl.SelectedIndexChanged
        _activeTab = CType(sender, TabControl).SelectedTab
        InitVisibleButtons()
    End Sub

    Public Shared Function GenerateMd5Hash(input As String) As String
        Dim x = New Cryptography.MD5CryptoServiceProvider()
        Dim computeHash = System.Text.Encoding.UTF8.GetBytes(input)
        computeHash = x.ComputeHash(computeHash)
        Dim stringBuilder = New System.Text.StringBuilder()
        For Each var As Byte In computeHash
            stringBuilder.Append(var.ToString("x2").ToLower())
        Next
        Return stringBuilder.ToString()
    End Function

    Private Sub SaveConfigurationBtn_Click(sender As Object, e As EventArgs) Handles SaveConfigurationBtn.Click
        If Not _activeTab.Name = "Custom" Then
            Exit Sub
        End If

        If ConfigurationNameTB.TextLength = 0 Then
            SettingsErrorProvider.SetError(ConfigurationNameTB, "Es muss ein Name für die Konfiguration angegeben werden")
            ConfigurationNameTB.Focus()
            Exit Sub
        Else
            SettingsErrorProvider.SetError(ConfigurationNameTB, "")
        End If

        If uGameTimeMTB.Text.Length <= 3 Then
            SettingsErrorProvider.SetError(uGameTimeMTB, "Es muss eine Gesamtspielzeit angegeben werden")
            uGameTimeMTB.Focus()
            Exit Sub
        Else
            SettingsErrorProvider.SetError(uGameTimeMTB, "")
        End If

        If uHalfTimeMTB.Text.Length <= 3 Then
            SettingsErrorProvider.SetError(uHalfTimeMTB, "Es muss die Halbzeit der Gesamtspielzeit angegeben werden")
            uHalfTimeMTB.Focus()
            Exit Sub
        Else
            SettingsErrorProvider.SetError(uHalfTimeMTB, "")
        End If

        If uGamePeriodsUD.Value = 0 Then
            SettingsErrorProvider.SetError(uGamePeriodsUD, "Es müssen die Anzahl der Halbzeiten angegeben werden")
            uGamePeriodsUD.Focus()
            Exit Sub
        Else
            SettingsErrorProvider.SetError(uGamePeriodsUD, "")
        End If

        If uTimeOutMTB.Text.Length <= 3 Then
            SettingsErrorProvider.SetError(uTimeOutMTB, "Es muss die Timeout Zeit angegeben werden")
            uTimeOutMTB.Focus()
            Exit Sub
        Else
            SettingsErrorProvider.SetError(uTimeOutMTB, "")
        End If

        If uExtraTimeMTB.Text.Length <= 3 Then
            SettingsErrorProvider.SetError(uExtraTimeMTB, "Es muss die Verlängerung als Spielzeit angegeben werden")
            uExtraTimeMTB.Focus()
            Exit Sub
        Else
            SettingsErrorProvider.SetError(uExtraTimeMTB, "")
        End If

        If uExtraHalfTimeMTB.Text.Length <= 3 Then
            SettingsErrorProvider.SetError(uExtraHalfTimeMTB, "Es muss die Halbzeit der Verlängerung als Spielzeit angegeben werden")
            uExtraHalfTimeMTB.Focus()
            Exit Sub
        Else
            SettingsErrorProvider.SetError(uExtraHalfTimeMTB, "")
        End If

        If uExtraRoundsUD.Value = 0 Then
            SettingsErrorProvider.SetError(uExtraRoundsUD, "Es muss die Anzahl der Verlängerungen angegeben werden")
            uExtraRoundsUD.Focus()
            Exit Sub
        Else
            SettingsErrorProvider.SetError(uExtraRoundsUD, "")
        End If

        Dim _conf As New Configuration(GenerateMd5Hash(ConfigurationNameTB.Text), ConfigurationNameTB.Text)
        Dim _settings As New Configuration.UserSetting() With {
            .GameTime = uGameTimeMTB.Text,
            .HalfTime = uHalfTimeMTB.Text,
            .TimeoutTime = uTimeOutMTB.Text,
            .GamePeriods = uGamePeriodsUD.Value,
            .ExtraTime = uExtraHalfTimeMTB.Text,
            .ExtraHalftime = uExtraHalfTimeMTB.Text,
            .ExtraTimeRounds = uExtraRoundsUD.Value
        }

        _conf.Settings = _settings
        Dim _jsStr As String = JsonConvert.SerializeObject(_conf)
        SaveConfiguration2File(_jsStr)
        readConfigFiles()
        initJsonSettings()
    End Sub


    Private Sub HideErrorNotification()
        With ConfigurationTSL
            .Text = ""
            .Visible = False
        End With
    End Sub

    Private Sub InitVisibleButtons()
        Select Case _activeTab.Name
            Case "General"
                SaveConfigurationBtn.Visible = False
                ConfigurationTSL.Visible = False
            Case "Custom"
                SaveConfigurationBtn.Visible = True
        End Select
    End Sub

    Private Sub SaveConfiguration2File(ByVal content As String)
        Dim _confFile As String = GenerateMd5Hash(ConfigurationNameTB.Text) & ".sbs"
        Dim _confPath As String = Path.Combine(My.Application.Info.DirectoryPath, "data", _confFile)

        Try
            Dim _str As Stream = IO.File.OpenWrite(_confPath)
            Dim _strW As New StreamWriter(_str, System.Text.Encoding.UTF8)
            _strW.Write(content)
            _strW.Flush()
            _strW.Close()
            _str.Close()
        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Exclamation)
        End Try
    End Sub

    Private Sub AddConfigurationBtn_Click(sender As Object, e As EventArgs) Handles AddConfigurationBtn.Click
        UserConfigurationsLB.SelectedIndex = -1
        ClearUserConfigurationFields()
    End Sub

    Private Sub EditConfigurationBtn_Click(sender As Object, e As EventArgs) Handles EditConfigurationBtn.Click

        If editConfiguration = False Then

            EditConfigurationBtn.BackgroundImage = My.Resources.reload_btn
            editConfiguration = True
            Dim conf As Configuration = UserConfigurationsLB.SelectedItem
            ConfigurationNameTB.Text = conf.Name
            ConfigurationNameTB.ReadOnly = True
            uGameTimeMTB.Text = conf.Settings.gameTime
            uHalfTimeMTB.Text = conf.Settings.halfTime
            uTimeOutMTB.Text = conf.Settings.timeoutTime
            uGamePeriodsUD.Value = CInt(conf.Settings.gamePeriods)
            uExtraTimeMTB.Text = conf.Settings.extraTime
            uExtraHalfTimeMTB.Text = conf.Settings.extraHalftime
            uExtraRoundsUD.Value = CInt(conf.Settings.extraTimeRounds)

        Else
            ConfigurationNameTB.ReadOnly = False
            editConfiguration = False
            EditConfigurationBtn.BackgroundImage = My.Resources.edit_btn
            ClearUserConfigurationFields()
        End If

    End Sub

    Private Sub ClearUserConfigurationFields()
        ConfigurationNameTB.Text = ""
        uGameTimeMTB.Text = ""
        uHalfTimeMTB.Text = ""
        uTimeOutMTB.Text = ""
        uGamePeriodsUD.Value = 0
        uExtraHalfTimeMTB.Text = ""
        uExtraHalfTimeMTB.Text = ""
        uExtraRoundsUD.Value = 0
    End Sub

    Private Sub RemoveConfigurationBtn_Click(sender As Object, e As EventArgs) Handles RemoveConfigurationBtn.Click
        If UserConfigurationsLB.SelectedIndex = -1 Then
            MsgBox("Es wurde keine Konfiguration ausgewählt", MsgBoxStyle.Information, "Benutzerkonfiguration entfernen")
            Return
        End If

        Try
            Dim conf As Configuration = UserConfigurationsLB.SelectedItem
            Dim confFile As String = Path.Combine(My.Application.Info.DirectoryPath, "data", conf.UID & ".sbs")
            If File.Exists(confFile) Then
                File.Delete(confFile)
            End If
            UserConfigurationsLB.Items.Remove(conf)
            UserConfigurationsLB.SelectedIndex = -1
            EditConfigurationBtn.BackgroundImage = My.Resources.edit_btn
            ClearUserConfigurationFields()
            readConfigFiles()
            initJsonSettings()
        Catch ex As Exception
            MsgBox(ex.Message, MsgBoxStyle.Exclamation)
        End Try
    End Sub
End Class